@@ -43,6 +43,7 @@ import java.awt.geom.RoundRectangle2D;
 import java.awt.image.BufferedImage;
 import java.awt.image.RescaleOp;
 import java.awt.image.VolatileImage;
+import java.util.ArrayList;
 import java.util.EnumMap;
 import java.util.List;
 import java.util.logging.Level;
@@ -72,6 +73,10 @@ import net.sf.freecol.common.model.Tile;
 import net.sf.freecol.common.model.Turn;
 import net.sf.freecol.common.model.Unit;
 import net.sf.freecol.common.option.GameOptions;
+import net.sf.freecol.server.ai.AIObject;
+import net.sf.freecol.server.ai.EuropeanAIPlayer;
+import net.sf.freecol.server.ai.military.DefensiveMap;
+import net.sf.freecol.server.ai.military.DefensiveZone;
 
 
 /**
@@ -136,6 +141,8 @@ public final class MapViewer extends FreeColClientHolder {
      */
     private TileBounds tileBounds = new TileBounds(new Dimension(0, 0), 1f);
     
+    private List<Rectangle> fullyRepaintedAreas = new ArrayList<>();
+    
 
     /**
      * The constructor to use.
@@ -223,18 +230,17 @@ public final class MapViewer extends FreeColClientHolder {
     public boolean displayMap(Graphics2D g2d, Dimension size) {
         final long startMs = now();
         final Rectangle clipBounds = g2d.getClipBounds();
-        
         if (mapViewerBounds.getFocus() == null) {
             paintBlackBackground(g2d, clipBounds);
             return false;
         }
-        
+
         if (rpm.isRepaintsBlocked(size)) {
             final VolatileImage backBufferImage = rpm.getBackBufferImage();
             g2d.drawImage(backBufferImage, 0, 0, null);
             return false;
         }
-        
+
         boolean fullMapRenderedWithoutUsingBackBuffer = rpm.prepareBuffers(mapViewerBounds, mapViewerBounds.getFocus());
         final Rectangle dirtyClipBounds = rpm.getDirtyClipBounds();
         if (rpm.isAllDirty()) {
@@ -249,7 +255,12 @@ public final class MapViewer extends FreeColClientHolder {
 
         final Map map = getMap();
 
-        final Rectangle allRenderingClipBounds = clipBounds.union(dirtyClipBounds);
+        final Rectangle allRenderingClipBounds;
+        if (dirtyClipBounds.isEmpty()) {
+            allRenderingClipBounds = clipBounds;
+        } else {
+            allRenderingClipBounds = clipBounds.union(dirtyClipBounds);
+        }
         paintBlackBackground(backBufferG2d, allRenderingClipBounds);
         
         // Display the animated base tiles:
@@ -262,17 +273,14 @@ public final class MapViewer extends FreeColClientHolder {
         // Display everything else:
         final long animatedBaseMs = now();
         if (!dirtyClipBounds.isEmpty()) {
-            final TileClippingBounds tcb = new TileClippingBounds(map, dirtyClipBounds);
-            final Graphics2D nonAnimationG2d = nonAnimationBufferImage.createGraphics();
-            displayNonAnimationImages(nonAnimationG2d, dirtyClipBounds, tcb);
-            nonAnimationG2d.dispose();
+            displayToNonAnimationBufferImage(dirtyClipBounds, nonAnimationBufferImage, map);
         }
         
         final long nonAnimatedMs = now();   
         backBufferG2d.setTransform(backBufferOriginTransform);
         backBufferG2d.setClip(allRenderingClipBounds);
         backBufferG2d.drawImage(nonAnimationBufferImage, 0, 0, null);
-        backBufferG2d.dispose();
+        backBufferG2d.dispose();        
 
         g2d.drawImage(backBufferImage, 0, 0, null);      
         final long useBuffersMs = now();
@@ -302,6 +310,15 @@ public final class MapViewer extends FreeColClientHolder {
         mapViewerState.getChatDisplay().display(g2d, mapViewerBounds.getSize());
         final long chatMs = now();
 
+        if (FreeColDebugger.debugRendering()) {
+            fullyRepaintedAreas.add(dirtyClipBounds);
+            g2d.setColor(new Color(255, 0, 0, 100));
+            for (Rectangle r : fullyRepaintedAreas) {
+                g2d.fill(r);
+            }
+            fullyRepaintedAreas.clear();
+        }
+        
         verifyAndMarkAsClean(size, clipBounds);
         
         /*
@@ -329,6 +346,45 @@ public final class MapViewer extends FreeColClientHolder {
         return fullMapRenderedWithoutUsingBackBuffer;
     }
 
+    /**
+     * Paints the dirty tiles to the buffers. The screen will
+     * be updated the next time {@link #displayMap(Graphics2D, Dimension)}
+     * gets called.
+     * 
+     * This is used for handling several small dirty areas that would otherwise
+     * force a full repaint since the bounding box covers the entire screen.
+     * An example of this is diagonal scrolling.
+     */
+    public void paintImmediatelyToBuffersOnly() {
+        if (mapViewerBounds.getFocus() == null
+                || rpm.isRepaintsBlocked(mapViewerBounds.getSize())) {
+            return;
+        }
+        
+        rpm.prepareBuffers(mapViewerBounds, mapViewerBounds.getFocus());
+        final Rectangle dirtyClipBounds = rpm.getDirtyClipBounds();
+        
+        if (dirtyClipBounds.isEmpty()) {
+            return;
+        }
+        
+        final BufferedImage nonAnimationBufferImage = rpm.getNonAnimationBufferImage();
+        final Map map = getMap();
+        displayToNonAnimationBufferImage(dirtyClipBounds, nonAnimationBufferImage, map);
+        if (FreeColDebugger.debugRendering()) {
+            fullyRepaintedAreas.add(dirtyClipBounds);
+        }
+        rpm.markAsClean();
+    }
+
+    private void displayToNonAnimationBufferImage(final Rectangle dirtyClipBounds, final BufferedImage nonAnimationBufferImage,
+            final Map map) {
+        final TileClippingBounds tcb = new TileClippingBounds(map, dirtyClipBounds);
+        final Graphics2D nonAnimationG2d = nonAnimationBufferImage.createGraphics();
+        displayNonAnimationImages(nonAnimationG2d, dirtyClipBounds, tcb);
+        nonAnimationG2d.dispose();
+    }
+
     private void displayNonAnimationImages(Graphics2D nonAnimationG2d,
             Rectangle clipBounds,
             TileClippingBounds tcb) {
@@ -447,7 +503,7 @@ public final class MapViewer extends FreeColClientHolder {
         nonAnimationG2d.setColor(Color.BLACK);
         final boolean revengeMode = getGame().isInRevengeMode();
         if (!revengeMode) {
-            paintEachTile(nonAnimationG2d, tcb, (tileG2d, tile) -> {
+            paintEachTile(nonAnimationG2d, tcb.getTopLeftDirtyTile(), tcb.getUnitTiles(), (tileG2d, tile) -> {
                 final Unit unit = mapViewerState.findUnitInFront(tile);
                 if (unit == null || mapViewerState.getUnitAnimator().isOutForAnimation(unit)) {
                     return;
@@ -483,6 +539,8 @@ public final class MapViewer extends FreeColClientHolder {
             			null);
             }
         });
+        
+        displayDebugAiDefensiveMap(nonAnimationG2d, tcb);
 
         // Display the colony names, if needed
         long t14 = now();
@@ -530,6 +588,34 @@ public final class MapViewer extends FreeColClientHolder {
         }
     }
 
+    private void displayDebugAiDefensiveMap(Graphics2D nonAnimationG2d, TileClippingBounds tcb) {
+        if (FreeColDebugger.debugShowDefenceMapForPlayer() != null
+                && getFreeColServer() != null
+                && getFreeColServer().getAIMain() != null) {
+            final AIObject aiObject = getFreeColServer().getAIMain().getAIObject(FreeColDebugger.debugShowDefenceMapForPlayer().getId());
+            if (aiObject != null && aiObject instanceof EuropeanAIPlayer) {
+                final EuropeanAIPlayer eap = (EuropeanAIPlayer) aiObject;
+                final DefensiveMap defensiveMap = DefensiveMap.createDefensiveMap(eap);
+                paintEachTile(nonAnimationG2d, tcb.getTopLeftDirtyTile(), tcb.getBaseTiles(), (tileG2d, tile) -> {
+                    final DefensiveZone defensiveZone = defensiveMap.getDefensiveZone(tile);
+                    if (defensiveZone == null) {
+                        return;
+                    }
+                    if (defensiveZone.getNumberOfMilitaryEnemies() > 0) {
+                        tileG2d.setColor(new Color(255, 0, 0, 150));
+                    } else if (defensiveZone.isEnemiesInNeighbour()) {
+                        tileG2d.setColor(new Color(255, 100, 100, 150));
+                    } else if (defensiveZone.isExposed()) {
+                        tileG2d.setColor(new Color(255, 255, 0, 150));
+                    } else {
+                        tileG2d.setColor(new Color(0, 255, 0, 150));
+                    }
+                    tileG2d.fill(mapViewerScaledUtils.getFog());
+                });
+            }
+        }
+    }
+
     private boolean shouldFogOfWarBeDisplayed(final Player player, final ClientOptions options) {
         return player != null && getSpecification().getBoolean(GameOptions.FOG_OF_WAR) && options.getBoolean(ClientOptions.DISPLAY_FOG_OF_WAR);
     }
@@ -1079,6 +1165,7 @@ public final class MapViewer extends FreeColClientHolder {
     private void verifyAndMarkAsClean(Dimension size, final Rectangle clipBounds) {
         final Rectangle entireScreen = new Rectangle(0, 0, size.width, size.height);
         final Rectangle relevantDirtyClipBounds = rpm.getDirtyClipBounds().intersection(entireScreen);
+
         if (relevantDirtyClipBounds.isEmpty() || clipBounds.contains(relevantDirtyClipBounds)) {
             rpm.markAsClean();
         } else {
@@ -1266,6 +1353,7 @@ public final class MapViewer extends FreeColClientHolder {
         private final int clipTopY;
         
         private final List<Tile> baseTiles;
+        private final List<Tile> unitTiles;
         private final List<Tile> extendedTiles;
         private final List<Tile> superExtendedTiles;
         
@@ -1297,6 +1385,12 @@ public final class MapViewer extends FreeColClientHolder {
                     subMapWidth,
                     subMapHeight);
             
+            unitTiles = map.subMap(
+                    topLeftDirtyTile.getX(),
+                    topLeftDirtyTile.getY(),
+                    subMapWidth,
+                    subMapHeight + 1);
+            
             extendedTiles = map.subMap(
                     topLeftDirtyTile.getX(),
                     topLeftDirtyTile.getY() - 1,
@@ -1328,6 +1422,16 @@ public final class MapViewer extends FreeColClientHolder {
             return baseTiles;
         }
         
+        /**
+         * The tiles to be repainted for unit graphics that might extend
+         * up to half a tile in height above the base tile.
+         *
+         * @return The list of tiles to repaint.
+         */
+        public List<Tile> getUnitTiles() {
+            return unitTiles;
+        }
+        
         /**
          * The tiles to be repainted for graphics that might have
          * double height compared to the tile size.
