@@ -23,12 +23,11 @@ import java.awt.event.ActionEvent;
 import java.util.List;
 import java.util.logging.Logger;
 
+import javax.swing.AbstractAction;
 import javax.swing.ImageIcon;
-
 import javax.swing.JButton;
 
 import net.miginfocom.swing.MigLayout;
-
 import net.sf.freecol.client.FreeColClient;
 import net.sf.freecol.common.i18n.Messages;
 import net.sf.freecol.common.model.AbstractUnit;
@@ -111,7 +110,14 @@ public final class RecruitPanel extends FreeColPanel {
 
         okButton.setText(Messages.message("close"));
         add(okButton, "newline 20, tag ok");
-
+        
+        setEscapeAction(new AbstractAction() {
+            @Override
+            public void actionPerformed(ActionEvent ae) {
+                okButton.doClick();
+            }
+        });
+        
         setSize(getPreferredSize());
         revalidate();
     }
