@@ -19,10 +19,14 @@
 
 package net.sf.freecol.client.gui.panel;
 
+import static net.sf.freecol.common.util.CollectionUtils.count;
+
 import java.awt.Component;
+import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
 import java.util.logging.Logger;
 
+import javax.swing.AbstractAction;
 import javax.swing.JButton;
 import javax.swing.JCheckBox;
 import javax.swing.JScrollPane;
@@ -31,19 +35,16 @@ import javax.swing.JTextField;
 import javax.swing.ScrollPaneConstants;
 
 import net.miginfocom.swing.MigLayout;
-
 import net.sf.freecol.client.FreeColClient;
 import net.sf.freecol.client.gui.GUI;
 import net.sf.freecol.client.gui.panel.FreeColButton.ButtonStyle;
 import net.sf.freecol.common.i18n.Messages;
+import net.sf.freecol.common.model.Game.LogoutReason;
 import net.sf.freecol.common.model.NationOptions;
 import net.sf.freecol.common.model.NationOptions.NationState;
 import net.sf.freecol.common.model.Specification;
-import net.sf.freecol.common.model.Game.LogoutReason;
 import net.sf.freecol.common.option.GameOptions;
 import net.sf.freecol.common.option.MapGeneratorOptions;
-import net.sf.freecol.common.option.OptionGroup;
-import static net.sf.freecol.common.util.CollectionUtils.*;
 
 
 /**
@@ -143,12 +144,8 @@ public final class StartGamePanel extends FreeColPanel {
         }
 
         NationOptions nationOptions = getGame().getNationOptions();
-
         cancel = Utility.localizedButton("cancel");
-        setCancelComponent(cancel);
-
         JScrollPane chatScroll = null, tableScroll;
-
         table = new PlayersTable(getFreeColClient(), nationOptions,
                                  getMyPlayer());
 
@@ -211,6 +208,13 @@ public final class StartGamePanel extends FreeColPanel {
         gameOptions.addActionListener(gameOptionsCmd);
         mapGeneratorOptions.addActionListener(mapGeneratorOptionsCmd);
 
+        setEscapeAction(new AbstractAction() {
+            @Override
+            public void actionPerformed(ActionEvent ae) {
+                cancel.doClick();
+            }
+        });
+        
         setEnabled(true);
     }
 
