@@ -45,6 +45,7 @@ import java.util.logging.Level;
 import java.util.logging.Logger;
 import java.util.regex.Pattern;
 import java.util.stream.Collectors;
+import java.util.stream.Stream;
 
 import javax.swing.filechooser.FileFilter;
 import javax.swing.filechooser.FileNameExtensionFilter;
@@ -231,7 +232,16 @@ public class FreeColDataFile {
         if (uri == null) {
             return;
         }
-        final Resource resource = resourceFactory.createResource(key, uri);
+        
+        /*
+         * The caching key should be (and only be) the same when it's the
+         * same actual resource data (for example, the same image file).
+         * 
+         * Please note that the key is not suitable for a caching key,
+         * since the resource might get replaced by a mod.
+         */
+        final String cachingKey = uri.toString();
+        final Resource resource = resourceFactory.createResource(key, cachingKey, uri);
         
         /*
          * Rivers need new keys in order to support variations.
@@ -318,7 +328,7 @@ public class FreeColDataFile {
             .stream()
             .filter(entry -> entry.getKey() != null)
             .forEach(entry -> {
-                final ImageResource variationResource = (ImageResource) resourceFactory.createResource(imageResource.getPrimaryKey(), entry.getKey());
+                final ImageResource variationResource = (ImageResource) resourceFactory.createResource("", imageResource.getCachingKey(), entry.getKey());
                 if (variationResource != null) {
                     variationResource.addAlternativeResourceLocators(entry.getValue());
                     imageResource.addVariation(variationResource);
@@ -388,12 +398,12 @@ public class FreeColDataFile {
         final String suffix = resourceFilename.substring(resourceFilename.lastIndexOf("."));
         final String completeRegex = Pattern.quote(prefix) + regex + Pattern.quote(suffix);
         
-        final List<Path> paths = Files.list(filePath.getParent())
-                .sorted()
-                .filter(p -> p.getFileName().toString().matches(completeRegex) && (!findVariation || !p.equals(filePath)))
-                .collect(Collectors.toList());
-        
-        return paths;
+        try (Stream<Path> pathStream = Files.list(filePath.getParent())) {
+            return pathStream
+                    .sorted()
+                    .filter(p -> p.getFileName().toString().matches(completeRegex) && (!findVariation || !p.equals(filePath)))
+                    .collect(Collectors.toList());
+        }
     }
 
     /**
