@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2015   The FreeCol Team
+ *  Copyright (C) 2002-2022   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -24,16 +24,16 @@ import javax.xml.stream.XMLStreamException;
 import net.sf.freecol.common.io.FreeColXMLReader;
 import net.sf.freecol.common.io.FreeColXMLWriter;
 
-import org.w3c.dom.Element;
-
 
 /**
- * The <code>Ability</code> class encapsulates a bonus or penalty
+ * The {@code Ability} class encapsulates a bonus or penalty
  * that can be applied to any action within the game, most obviously
  * combat.
  */
 public final class Ability extends Feature {
 
+    public static final String TAG = "ability";
+
     /**
      * The ability to add the current tax as a bonus to the production
      * of bells.  Provided by the {@link FoundingFather} Thomas Paine.
@@ -139,6 +139,10 @@ public final class Ability extends Feature {
     public static final String CAN_BE_EQUIPPED
         = "model.ability.canBeEquipped";
 
+    /** The ability of certain REF units to surrender on defeat. */
+    public static final String CAN_BE_SURRENDERED
+        = "model.ability.canBeSurrendered";
+
     /** The ability of a player to recruit units. */
     public static final String CAN_RECRUIT_UNIT
         = "model.ability.canRecruitUnit";
@@ -177,13 +181,6 @@ public final class Ability extends Feature {
     public static final String COASTAL_ONLY
         = "model.ability.coastalOnly";
 
-    /**
-     * The ability of certain consumers (e.g. BuildQueues) to consume
-     * a large amount of goods at once instead of turn by turn.
-     */
-    public static final String CONSUME_ALL_OR_NOTHING
-        = "model.ability.consumeAllOrNothing";
-
     /** The ability of customs houses to trade with other players. */
     public static final String CUSTOM_HOUSE_TRADES_WITH_FOREIGN_COUNTRIES
         = "model.ability.customHouseTradesWithForeignCountries";
@@ -404,13 +401,41 @@ public final class Ability extends Feature {
     public static final String UNDEAD
         = "model.ability.undead";
 
+    /** Upgrade converts to free colonist with Casas. */
+    public static final String UPGRADE_CONVERT
+        = "model.ability.upgradeConvert";
+    
 
     /** The ability value. */
     private boolean value = true;
 
 
     /**
-     * Creates a new <code>Ability</code> instance.
+     * Deliberately trivial constructor.
+     *
+     * @param specification The {@code Specification} to use.
+     */
+    public Ability(Specification specification) {
+        super(specification);
+    }
+
+    /**
+     * Creates a new {@code Ability} instance.
+     *
+     * @param id The object identifier.
+     * @param source The source {@code FreeColSpecObjectType}.
+     * @param value The ability value.
+     */
+    public Ability(String id, FreeColSpecObjectType source, boolean value) {
+        this((source == null) ? null : source.getSpecification());
+
+        setId(id);
+        setSource(source);
+        this.value = value;
+    }
+
+    /**
+     * Creates a new {@code Ability} instance.
      *
      * @param id The object identifier.
      */
@@ -419,7 +444,7 @@ public final class Ability extends Feature {
     }
 
     /**
-     * Creates a new <code>Ability</code> instance.
+     * Creates a new {@code Ability} instance.
      *
      * @param id The object identifier.
      * @param value The ability value.
@@ -429,50 +454,29 @@ public final class Ability extends Feature {
     }
 
     /**
-     * Creates a new <code>Ability</code> instance.
+     * Creates a new {@code Ability} instance.
      *
-     * @param id The object identifier.
-     * @param source The source <code>FreeColGameObjectType</code>.
-     * @param value The ability value.
-     */
-    public Ability(String id, FreeColGameObjectType source, boolean value) {
-        setId(id);
-        setSource(source);
-        this.value = value;
-    }
-
-    /**
-     * Creates a new <code>Ability</code> instance.
-     *
-     * @param template An <code>Ability</code> to copy from.
+     * @param template An {@code Ability} to copy from.
      */
     public Ability(Ability template) {
-        copyFrom(template);
-        this.value = template.value;
+        this((Specification)null);
+        
+        setId(template.getId());
+        copyIn(template);
     }
 
     /**
-     * Creates a new <code>Ability</code> instance.
+     * Creates a new {@code Ability} instance.
      *
-     * @param xr The <code>FreeColXMLReader</code> to read from.
-     * @param specification A <code>Specification</code> to refer to.
+     * @param xr The {@code FreeColXMLReader} to read from.
+     * @param specification A {@code Specification} to refer to.
      * @exception XMLStreamException if an error occurs
      */
     public Ability(FreeColXMLReader xr,
                    Specification specification) throws XMLStreamException {
-        setSpecification(specification);
-        readFromXML(xr);
-    }
+        this(specification);
 
-    /**
-     * Creates a new <code>Ability</code> instance.
-     *
-     * @param e The <code>Element</code> to read from.
-     * @param specification A <code>Specification</code> to refer to.
-     */
-    public Ability(Element e, Specification specification) {
-        setSpecification(specification);
-        readFromXMLElement(e);
+        readFromXML(xr);
     }
 
 
@@ -495,32 +499,20 @@ public final class Ability extends Feature {
     }
 
 
-    // Override Object
+    // Override FreeColObject
 
     /**
      * {@inheritDoc}
      */
     @Override
-    public boolean equals(Object o) {
-        if (this == o) return true;
-        if (o instanceof Ability) {
-            return this.value == ((Ability)o).value
-                && super.equals(o);
-        }
-        return false;
+    public <T extends FreeColObject> boolean copyIn(T other) {
+        Ability o = copyInCast(other, Ability.class);
+        if (o == null || !super.copyIn(o)) return false;
+        this.value = o.getValue();
+        return true;
     }
 
-    /**
-     * {@inheritDoc}
-     */
-    @Override
-    public int hashCode() {
-        int hash = super.hashCode();
-        hash += (value) ? 1 : 0;
-        return hash;
-    }
-
-
+    
     // Serialization
 
 
@@ -544,32 +536,49 @@ public final class Ability extends Feature {
         value = xr.getAttribute(VALUE_TAG, true);
     }
 
+    /**
+     * {@inheritDoc}
+     */
+    public String getXMLTagName() { return TAG; }
+
+
+    // Override Object
+
     /**
      * {@inheritDoc}
      */
     @Override
-    public String toString() {
-        StringBuilder sb = new StringBuilder(32);
-        sb.append("[ ").append(getId());
-        if (getSource() != null) {
-            sb.append(" (").append(getSource().getId()).append(")");
+    public boolean equals(Object o) {
+        if (this == o) return true;
+        if (o instanceof Ability) {
+            Ability other = (Ability)o;
+            return this.value == other.value
+                && super.equals(other);
         }
-        sb.append(" = ").append(value).append(" ]");
-        return sb.toString();
+        return false;
     }
 
     /**
      * {@inheritDoc}
      */
     @Override
-    public String getXMLTagName() { return getXMLElementTagName(); }
+    public int hashCode() {
+        int hash = super.hashCode();
+        hash += (value) ? 1 : 0;
+        return hash;
+    }
 
     /**
-     * Gets the XML tag name for this element.
-     *
-     * @return "ability".
+     * {@inheritDoc}
      */
-    public static String getXMLElementTagName() {
-        return "ability";
+    @Override
+    public String toString() {
+        StringBuilder sb = new StringBuilder(32);
+        sb.append("[ ").append(getId());
+        if (getSource() != null) {
+            sb.append(" (").append(getSource().getId()).append(')');
+        }
+        sb.append(" = ").append(value).append(" ]");
+        return sb.toString();
     }
 }
