@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2015   The FreeCol Team
+ *  Copyright (C) 2002-2022   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -30,8 +30,7 @@ import javax.xml.stream.XMLStreamException;
 
 import net.sf.freecol.common.io.FreeColXMLReader;
 import net.sf.freecol.common.io.FreeColXMLWriter;
-
-import org.w3c.dom.Element;
+import static net.sf.freecol.common.util.CollectionUtils.*;
 
 
 /**
@@ -43,14 +42,18 @@ public final class Market extends FreeColGameObject implements Ownable {
 
     private static final Logger logger = Logger.getLogger(Market.class.getName());
 
+    public static final String TAG = "market";
+
     /**
      * European markets are bottomless.  Goods present never decrease
      * below this threshold.
      */
     public static final int MINIMUM_AMOUNT = 100;
 
+    public static final String PRICE_CHANGE = "priceChange";
+    
     /**
-     * Constant for specifying the access to this <code>Market</code>
+     * Constant for specifying the access to this {@code Market}
      * when selling goods.
      */
     public static enum Access {
@@ -58,19 +61,25 @@ public final class Market extends FreeColGameObject implements Ownable {
         CUSTOM_HOUSE,
     }
 
+
     /** The contents of the market, keyed by goods type. */
     private final Map<GoodsType, MarketData> marketData = new HashMap<>();
 
     /** The owning player. */
     private Player owner;
 
-    /** Watching listeners.  Do not serialize. */
+    // Do not serialize below
+
+    /** Watching listeners. */
     private final ArrayList<TransactionListener> transactionListeners
         = new ArrayList<>();
 
 
     /**
      * Main constructor for creating a market for a new player.
+     *
+     * @param game The enclosing {@code Game}.
+     * @param player The {@code Player} to own the market.
      */
     public Market(Game game, Player player) {
         super(game);
@@ -88,12 +97,11 @@ public final class Market extends FreeColGameObject implements Ownable {
     }
 
     /**
-     * Creates a new <code>Market</code> with the given identifier.
-     * The object should later be initialized by calling either
-     * {@link #readFromXML(FreeColXMLReader)} or
-     * {@link #readFromXMLElement(Element)}.
+     * Creates a new {@code Market} with the given identifier.
      *
-     * @param game The enclosing <code>Game</code>.
+     * The object should be initialized later.
+     *
+     * @param game The enclosing {@code Game}.
      * @param id The object identifier.
      */
     public Market(Game game, String id) {
@@ -101,16 +109,60 @@ public final class Market extends FreeColGameObject implements Ownable {
     }
 
 
+    /**
+     * Get the market data.
+     *
+     * @return The map of goods type to market data.
+     */
+    private Map<GoodsType, MarketData> getMarketData() {
+        synchronized (this.marketData) {
+            return this.marketData;
+        }
+    }
+
+    /**
+     * Get the market data values.
+     *
+     * @return The market data in this market.
+     */
+    public Collection<MarketData> getMarketDataValues() {
+        synchronized (this.marketData) {
+            return this.marketData.values();
+        }
+    }
+
+    /**
+     * Gets the market data for a type of goods.
+     *
+     * Public so the server can send individual MarketData updates.
+     *
+     * @param goodsType The {@code GoodsType} to look for.
+     * @return The corresponding {@code MarketData}, or null if none.
+     */
+    public MarketData getMarketData(GoodsType goodsType) {
+        synchronized (this.marketData) {
+            return this.marketData.get(goodsType);
+        }
+    }
+
+    /**
+     * Set the market data for a given goods type.
+     *
+     * @param goodsType The {@code GoodsType} to set the market data for.
+     * @param data The new {@code MarketData}.
+     */
     private void putMarketData(GoodsType goodsType, MarketData data) {
-        marketData.put(goodsType, data);
+        synchronized (this.marketData) {
+            this.marketData.put(goodsType, data);
+        }
     }
 
     /**
      * Gets the market data for a specified goods type, creating it
      * if it does not exist yet.
      *
-     * @param goodsType The <code>GoodsType</code> to query.
-     * @return The <code>MarketData</code> required.
+     * @param goodsType The {@code GoodsType} to query.
+     * @return The {@code MarketData} required.
      */
     private MarketData requireMarketData(GoodsType goodsType) {
         MarketData data = getMarketData(goodsType);
@@ -121,27 +173,25 @@ public final class Market extends FreeColGameObject implements Ownable {
         return data;
     }
 
-    // ------------------------------------------------------------ API methods
-
     /**
-     * Get the market data values.
+     * Set the market data.
      *
-     * @return The market data in this market.
+     * @param data The new market data.
      */
-    public Collection<MarketData> getMarketData() {
-        return this.marketData.values();
+    private void setMarketData(Map<GoodsType, MarketData> data) {
+        synchronized (this.marketData) {
+            this.marketData.clear();
+            this.marketData.putAll(data);
+        }
     }
 
     /**
-     * Gets the market data for a type of goods.
-     *
-     * Public so the server can send individual MarketData updates.
-     *
-     * @param goodsType The <code>GoodsType</code> to look for.
-     * @return The corresponding <code>MarketData</code>, or null if none.
+     * Clear the market data.
      */
-    public MarketData getMarketData(GoodsType goodsType) {
-        return marketData.get(goodsType);
+    private void clearMarketData() {
+        synchronized (this.marketData) {
+            this.marketData.clear();
+        }
     }
 
     /**
@@ -158,7 +208,7 @@ public final class Market extends FreeColGameObject implements Ownable {
     /**
      * Determines the cost to buy a single unit of a particular type of good.
      *
-     * @param type A <code>GoodsType</code> value.
+     * @param type A {@code GoodsType} value.
      * @return The cost to buy a single unit of the given type of goods.
      */
     public int getCostToBuy(GoodsType type) {
@@ -170,7 +220,7 @@ public final class Market extends FreeColGameObject implements Ownable {
      * Determines the price paid for the sale of a single unit of a particular
      * type of goods.
      *
-     * @param type A <code>GoodsType</code> value.
+     * @param type A {@code GoodsType} value.
      * @return The price for a single unit of the given type of goods
      *         if it is for sale.
      */
@@ -182,7 +232,7 @@ public final class Market extends FreeColGameObject implements Ownable {
     /**
      * Add (or remove) some goods to this market.
      *
-     * @param goodsType The <code>GoodsType</code> to add.
+     * @param goodsType The {@code GoodsType} to add.
      * @param amount The amount of goods.
      * @return True if the price changes as a result of this addition.
      */
@@ -199,7 +249,7 @@ public final class Market extends FreeColGameObject implements Ownable {
     /**
      * Gets the initial price of a given goods type.
      *
-     * @param goodsType The <code>GoodsType</code> to get the initial price of.
+     * @param goodsType The {@code GoodsType} to get the initial price of.
      * @return The initial price.
      */
     public int getInitialPrice(GoodsType goodsType) {
@@ -210,7 +260,7 @@ public final class Market extends FreeColGameObject implements Ownable {
     /**
      * Sets the initial price of a given goods type.
      *
-     * @param goodsType The <code>GoodsType</code> to set the initial price of.
+     * @param goodsType The {@code GoodsType} to set the initial price of.
      * @param amount The new initial price.
      */
     public void setInitialPrice(GoodsType goodsType, int amount) {
@@ -219,9 +269,9 @@ public final class Market extends FreeColGameObject implements Ownable {
     }
 
     /**
-     * Gets the price of a given goods when the <code>Player</code> buys.
+     * Gets the price of a given goods when the {@code Player} buys.
      *
-     * @param type a <code>GoodsType</code> value
+     * @param type a {@code GoodsType} value
      * @param amount The amount of goods.
      * @return The bid price of the given goods.
      */
@@ -231,9 +281,9 @@ public final class Market extends FreeColGameObject implements Ownable {
     }
 
     /**
-     * Gets the price of a given goods when the <code>Player</code> sells.
+     * Gets the price of a given goods when the {@code Player} sells.
      *
-     * @param type a <code>GoodsType</code> value
+     * @param type a {@code GoodsType} value
      * @param amount The amount of goods.
      * @return The sale price of the given goods.
      */
@@ -243,19 +293,20 @@ public final class Market extends FreeColGameObject implements Ownable {
     }
 
     /**
-     * Gets the price of a given goods when the <code>Player</code> sells.
+     * Gets the price of a given goods when the {@code Player} sells.
      *
-     * @param goods a <code>Goods</code> value
-     * @return an <code>int</code> value
+     * @param <T> The base type of the goods.
+     * @param goods The {@code Goods} to evaluate.
+     * @return The price.
      */
-    public int getSalePrice(Goods goods) {
+    public <T extends AbstractGoods> int getSalePrice(T goods) {
         return getSalePrice(goods.getType(), goods.getAmount());
     }
 
     /**
      * Gets the arrears for of a given goods type.
      *
-     * @param goodsType The <code>GoodsType</code> to get arrears for.
+     * @param goodsType The {@code GoodsType} to get arrears for.
      * @return The arrears.
      */
     public int getArrears(GoodsType goodsType) {
@@ -266,7 +317,7 @@ public final class Market extends FreeColGameObject implements Ownable {
     /**
      * Sets the arrears associated with a type of goods.
      *
-     * @param goodsType The <code>GoodsType</code> to set the arrears for.
+     * @param goodsType The {@code GoodsType} to set the arrears for.
      * @param value The amount of arrears to set.
      */
     public void setArrears(GoodsType goodsType, int value) {
@@ -277,7 +328,7 @@ public final class Market extends FreeColGameObject implements Ownable {
     /**
      * Gets the sales of a type of goods.
      *
-     * @param goodsType The <code>GoodsType</code> to get the sales for.
+     * @param goodsType The {@code GoodsType} to get the sales for.
      * @return The current sales amount.
      */
     public int getSales(GoodsType goodsType) {
@@ -288,7 +339,7 @@ public final class Market extends FreeColGameObject implements Ownable {
     /**
      * Modifies the sales of a type of goods.
      *
-     * @param goodsType The <code>GoodsType</code> to set the sales for.
+     * @param goodsType The {@code GoodsType} to set the sales for.
      * @param amount The amount of sales to add to the current amount.
      */
     public void modifySales(GoodsType goodsType, int amount) {
@@ -302,7 +353,7 @@ public final class Market extends FreeColGameObject implements Ownable {
     /**
      * Gets the income before taxes for a type of goods.
      *
-     * @param goodsType The <code>GoodsType</code> to get the income for.
+     * @param goodsType The {@code GoodsType} to get the income for.
      * @return The current income before taxes.
      */
     public int getIncomeBeforeTaxes(GoodsType goodsType) {
@@ -313,7 +364,7 @@ public final class Market extends FreeColGameObject implements Ownable {
     /**
      * Modifies the income before taxes on sales of a type of goods.
      *
-     * @param goodsType The <code>GoodsType</code> to set the income for.
+     * @param goodsType The {@code GoodsType} to set the income for.
      * @param amount The amount of tax income to add to the current amount.
      */
     public void modifyIncomeBeforeTaxes(GoodsType goodsType, int amount) {
@@ -324,7 +375,7 @@ public final class Market extends FreeColGameObject implements Ownable {
     /**
      * Gets the income after taxes for a type of goods.
      *
-     * @param goodsType The <code>GoodsType</code> to get the income for.
+     * @param goodsType The {@code GoodsType} to get the income for.
      * @return The current income after taxes.
      */
     public int getIncomeAfterTaxes(GoodsType goodsType) {
@@ -335,7 +386,7 @@ public final class Market extends FreeColGameObject implements Ownable {
     /**
      * Modifies the income after taxes on sales of a type of goods.
      *
-     * @param goodsType The <code>GoodsType</code> to set the income for.
+     * @param goodsType The {@code GoodsType} to set the income for.
      * @param amount The amount of tax income to add to the current amount.
      */
     public void modifyIncomeAfterTaxes(GoodsType goodsType, int amount) {
@@ -346,7 +397,7 @@ public final class Market extends FreeColGameObject implements Ownable {
     /**
      * Gets the amount of a goods type in the market.
      *
-     * @param goodsType The <code>GoodsType</code> to get the amount of.
+     * @param goodsType The {@code GoodsType} to get the amount of.
      * @return The current amount of the goods in the market.
      */
     public int getAmountInMarket(GoodsType goodsType) {
@@ -379,10 +430,10 @@ public final class Market extends FreeColGameObject implements Ownable {
     }
 
     /**
-     * Make up a <code>ModelMessage</code> describing the change in this
-     * <code>Market</code> for a specified type of goods.
+     * Make up a {@code ModelMessage} describing the change in this
+     * {@code Market} for a specified type of goods.
      *
-     * @param goodsType The <code>GoodsType</code> that has changed price.
+     * @param goodsType The {@code GoodsType} that has changed price.
      * @return A message describing the change.
      */
     public ModelMessage makePriceChangeMessage(GoodsType goodsType) {
@@ -392,8 +443,8 @@ public final class Market extends FreeColGameObject implements Ownable {
         return (oldPrice == newPrice) ? null
             : new ModelMessage(ModelMessage.MessageType.MARKET_PRICES,
                                ((newPrice > oldPrice)
-                                ? "model.market.priceIncrease"
-                                : "model.market.priceDecrease"),
+                                   ? "model.market.priceIncrease"
+                                   : "model.market.priceDecrease"),
                                this, goodsType)
                 .addStringTemplate("%market%", owner.getMarketName())
                 .addNamed("%goods%", goodsType)
@@ -409,27 +460,30 @@ public final class Market extends FreeColGameObject implements Ownable {
      * not use during the game, the price change clamping mechanism should
      * remain in effect.
      *
-     * @param goodsType The <code>GoodsType</code> to update.
+     * @param goodsType The {@code GoodsType} to update.
      */
     public void update(GoodsType goodsType) {
         MarketData data = requireMarketData(goodsType);
-        if (data != null) data.update();
+        data.update();
     }
 
     /**
      * Get a sale price comparator for this market.
      *
-     * @return A suitable <code>Comparator</code>.
+     * @param <T> The {@code AbstractGoods} type to compare.
+     * @return A suitable {@code Comparator}.
      */
     public <T extends AbstractGoods> Comparator<T> getSalePriceComparator() {
         return Comparator.comparingInt((T t)
             -> getSalePrice(t.getType(), t.getAmount())).reversed();
     }
 
+    // TransactionListener support
+
     /**
      * Adds a transaction listener for notification of any transaction
      *
-     * @param listener The <code>TransactionListener</code> to add.
+     * @param listener The {@code TransactionListener} to add.
      */
     public void addTransactionListener(TransactionListener listener) {
         transactionListeners.add(listener);
@@ -438,7 +492,7 @@ public final class Market extends FreeColGameObject implements Ownable {
     /**
      * Removes a transaction listener
      *
-     * @param listener The <code>TransactionListener</code> to remove.
+     * @param listener The {@code TransactionListener} to remove.
      */
     public void removeTransactionListener(TransactionListener listener) {
         transactionListeners.remove(listener);
@@ -447,7 +501,7 @@ public final class Market extends FreeColGameObject implements Ownable {
     /**
      * Gets the listeners added to this market.
      *
-     * @return An array of all the <code>TransactionListener</code>s
+     * @return An array of all the {@code TransactionListener}s
      *     added, or an empty array if none found.
      */
     public TransactionListener[] getTransactionListener() {
@@ -458,9 +512,7 @@ public final class Market extends FreeColGameObject implements Ownable {
     // Interface Ownable
 
     /**
-     * Gets the owner of this <code>Market</code>.
-     *
-     * @return The owner of this <code>Market</code>.
+     * {@inheritDoc}
      */
     @Override
     public Player getOwner() {
@@ -468,9 +520,7 @@ public final class Market extends FreeColGameObject implements Ownable {
     }
 
     /**
-     * Sets the owner of this <code>Market</code>.
-     *
-     * @param owner The <code>Player</code> to own this <code>Market</code>.
+     * {@inheritDoc}
      */
     @Override
     public void setOwner(Player owner) {
@@ -488,6 +538,25 @@ public final class Market extends FreeColGameObject implements Ownable {
     }
 
 
+    // Overide FreeColObject
+
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public <T extends FreeColObject> boolean copyIn(T other) {
+        Market o = copyInCast(other, Market.class);
+        if (o == null || !super.copyIn(o)) return false;
+        final Game game = getGame();
+        this.marketData.clear();
+        for (MarketData md : game.update(o.getMarketDataValues(), true)) {
+            this.marketData.put(md.getGoodsType(), md);
+        }
+        this.owner = game.updateRef(o.getOwner());
+        return true;
+    }
+
+
     // Serialization
 
     private static final String OWNER_TAG = "owner";
@@ -512,7 +581,7 @@ public final class Market extends FreeColGameObject implements Ownable {
 
         if (xw.validFor(owner)) {
 
-            for (MarketData data : getSortedCopy(marketData.values())) {
+            for (MarketData data : sort(getMarketDataValues())) {
                 data.toXML(xw);
             }
         }
@@ -535,7 +604,7 @@ public final class Market extends FreeColGameObject implements Ownable {
     @Override
     protected void readChildren(FreeColXMLReader xr) throws XMLStreamException {
         // Clear containers.
-        marketData.clear();
+        clearMarketData();
 
         super.readChildren(xr);
     }
@@ -547,9 +616,8 @@ public final class Market extends FreeColGameObject implements Ownable {
     protected void readChild(FreeColXMLReader xr) throws XMLStreamException {
         final String tag = xr.getLocalName();
 
-        if (MarketData.getXMLElementTagName().equals(tag)) {
-            MarketData data = xr.readFreeColGameObject(getGame(), 
-                                                       MarketData.class);
+        if (MarketData.TAG.equals(tag)) {
+            MarketData data = xr.readFreeColObject(getGame(), MarketData.class);
             putMarketData(data.getGoodsType(), data);
 
         } else {
@@ -560,30 +628,23 @@ public final class Market extends FreeColGameObject implements Ownable {
     /**
      * {@inheritDoc}
      */
-    @Override
-    public String toString() {
-        StringBuilder sb = new StringBuilder(64);
-        sb.append("[").append(getId())
-            .append(" owner=").append(owner.getId());
-        for (MarketData md : getSortedCopy(marketData.values())) {
-            sb.append(" ").append(md);
-        }
-        sb.append("]");
-        return sb.toString();
-    }
+    public String getXMLTagName() { return TAG; }
+
+
+    // Override Object
 
     /**
      * {@inheritDoc}
      */
     @Override
-    public String getXMLTagName() { return getXMLElementTagName(); }
-
-    /**
-     * Gets the tag name of the root element representing this object.
-     *
-     * @return "market".
-     */
-    public static String getXMLElementTagName() {
-        return "market";
+    public String toString() {
+        StringBuilder sb = new StringBuilder(64);
+        sb.append('[').append(getId())
+            .append(" owner=").append((owner==null) ? "null" : owner.getId());
+        for (MarketData md : sort(getMarketDataValues())) {
+            sb.append(' ').append(md);
+        }
+        sb.append(']');
+        return sb.toString();
     }
 }
