@@ -35,7 +35,6 @@ import net.sf.freecol.common.model.Colony.NoBuildReason;
 import static net.sf.freecol.common.model.Constants.*;
 import static net.sf.freecol.common.util.CollectionUtils.*;
 
-
 /**
  * The various types of units in FreeCol.
  */
@@ -44,8 +43,7 @@ public final class UnitType extends BuildableType implements Consumer {
     public static final String TAG = "unit-type";
 
     /** Comparator for defence ability. */
-    public static final Comparator<UnitType> defenceComparator
-            = Comparator.comparingDouble(UnitType::getDefence);
+    public static final Comparator<UnitType> defenceComparator = Comparator.comparingDouble(UnitType::getDefence);
 
     /** The default offence value. */
     public static final int DEFAULT_OFFENCE = 0;
@@ -53,7 +51,6 @@ public final class UnitType extends BuildableType implements Consumer {
     /** The default offence value. */
     public static final int DEFAULT_DEFENCE = 1;
 
-
     /**
      * The offence of this UnitType. Only Units with an offence value
      * greater than zero can attack.
@@ -124,11 +121,10 @@ public final class UnitType extends BuildableType implements Consumer {
     /** The goods consumed per turn when in a settlement. */
     private TypeCountMap<GoodsType> consumption = null;
 
-
     /**
      * Creates a new {@code UnitType} instance.
      *
-     * @param id The object identifier.
+     * @param id            The object identifier.
      * @param specification The {@code Specification} to refer to.
      */
     public UnitType(String id, Specification specification) {
@@ -137,7 +133,6 @@ public final class UnitType extends BuildableType implements Consumer {
         this.defaultRole = specification.getDefaultRole();
     }
 
-
     /**
      * Get a key for the working as this unit type message.
      *
@@ -165,6 +160,15 @@ public final class UnitType extends BuildableType implements Consumer {
         return hasAbility(Ability.CARRY_GOODS);
     }
 
+    /**
+     * Can this unit type carry treasure?
+     * 
+     * @return True if unit can carry treasure
+     */
+    public boolean canCarryTreasure() {
+        return hasAbility(Ability.CARRY_TREASURE);
+    }
+
     /**
      * Gets the score for acquiring a unit of this type.
      *
@@ -223,7 +227,7 @@ public final class UnitType extends BuildableType implements Consumer {
      * Is this a defensive unit type?
      *
      * Default defence is 1, same a for colonists, thus to be defensive, a
-     * colonist must have a military role.  Artillery of all sorts has
+     * colonist must have a military role. Artillery of all sorts has
      * higher defense so they are automatically defensive.
      *
      * @return True if base defensive ability is greater than the default.
@@ -366,7 +370,7 @@ public final class UnitType extends BuildableType implements Consumer {
         this.mercenaryPrice = price;
     }
     // end @compat 0.11.6
-    
+
     /**
      * Get the base movement of this unit type.
      *
@@ -458,27 +462,31 @@ public final class UnitType extends BuildableType implements Consumer {
      *
      * @param teacherType The {@code UnitType} of the teacher.
      * @return The {@code UnitType} that this unit type can be educated
-     *     to by the teacher unit type, or null if education is not possible.
+     *         to by the teacher unit type, or null if education is not possible.
      */
     public UnitType getTeachingType(UnitType teacherType) {
         final Specification spec = getSpecification();
         final UnitType taught = teacherType.getSkillTaught();
         final int taughtLevel = taught.getSkill();
-        if (getSkill() >= taughtLevel) return null; // Fail fast
+        if (getSkill() >= taughtLevel)
+            return null; // Fail fast
 
         // Is there an education change that gets this unit type to the
-        // type taught by the teacher type?  If so, the taught type is valid
-        // and should be returned at once.  Accumulate other intermediate
+        // type taught by the teacher type? If so, the taught type is valid
+        // and should be returned at once. Accumulate other intermediate
         // changes that do not reach the taught type level.
         List<UnitType> todo = new ArrayList<>();
         for (UnitTypeChange uc : spec.getUnitChanges(UnitChangeType.EDUCATION, this)) {
-            if (uc.to == taught) return taught;
-            if (uc.to.getSkill() < taughtLevel) todo.add(uc.to);
+            if (uc.to == taught)
+                return taught;
+            if (uc.to.getSkill() < taughtLevel)
+                todo.add(uc.to);
         }
-        // Can the teacher teach any of the intermediate changes?  If so,
-        // that change is valid.  Otherwise, education is not possible.
+        // Can the teacher teach any of the intermediate changes? If so,
+        // that change is valid. Otherwise, education is not possible.
         for (UnitType ut : todo) {
-            if (ut.getTeachingType(teacherType) != null) return ut;
+            if (ut.getTeachingType(teacherType) != null)
+                return ut;
         }
         return null;
     }
@@ -572,7 +580,7 @@ public final class UnitType extends BuildableType implements Consumer {
     /**
      * Add consumption.
      *
-     * @param type The {@code GoodsType} to consume.
+     * @param type   The {@code GoodsType} to consume.
      * @param amount The amount of goods to consume.
      */
     private void addConsumption(GoodsType type, int amount) {
@@ -586,11 +594,11 @@ public final class UnitType extends BuildableType implements Consumer {
      * {@inheritDoc}
      */
     public NoBuildReason canBeBuiltInColony(Colony colony,
-                                            List<BuildableType> assumeBuilt) {
+            List<BuildableType> assumeBuilt) {
         // Non-person units need a BUILD ability, present or assumed.
         if (!hasAbility(Ability.PERSON)
-            && !colony.hasAbility(Ability.BUILD, this)
-            && none(assumeBuilt, bt -> bt.hasAbility(Ability.BUILD, this))) {
+                && !colony.hasAbility(Ability.BUILD, this)
+                && none(assumeBuilt, bt -> bt.hasAbility(Ability.BUILD, this))) {
             return Colony.NoBuildReason.MISSING_BUILD_ABILITY;
         }
         return Colony.NoBuildReason.NONE;
@@ -599,9 +607,11 @@ public final class UnitType extends BuildableType implements Consumer {
     @Override
     public int getMinimumIndex(Colony colony, JList<BuildableType> buildQueueList, int UNABLE_TO_BUILD) {
         ListModel<BuildableType> buildQueue = buildQueueList.getModel();
-        if (colony.canBuild(this)) return 0;
+        if (colony.canBuild(this))
+            return 0;
         for (int index = 0; index < buildQueue.getSize(); index++) {
-            if (buildQueue.getElementAt(index).hasAbility(Ability.BUILD, this)) return index + 1;
+            if (buildQueue.getElementAt(index).hasAbility(Ability.BUILD, this))
+                return index + 1;
         }
         return UNABLE_TO_BUILD;
     }
@@ -618,11 +628,13 @@ public final class UnitType extends BuildableType implements Consumer {
 
         // does not depend on anything, nothing depends on it
         // can be built at any time
-        if (canBuild) return buildQueueLastPos;
+        if (canBuild)
+            return buildQueueLastPos;
         // check for building in queue that allows builting this unit
         for (int index = 0; index < buildQueue.getSize(); index++) {
             BuildableType toBuild = buildQueue.getElementAt(index);
-            if (toBuild == this) continue;
+            if (toBuild == this)
+                continue;
             if (toBuild.hasAbility(Ability.BUILD, this)) {
                 return buildQueueLastPos;
             }
@@ -630,7 +642,6 @@ public final class UnitType extends BuildableType implements Consumer {
         return UNABLE_TO_BUILD;
     }
 
-
     // Interface Consumer
 
     /**
@@ -640,8 +651,8 @@ public final class UnitType extends BuildableType implements Consumer {
     public List<AbstractGoods> getConsumedGoods() {
         return (consumption == null) ? Collections.<AbstractGoods>emptyList()
                 : transform(consumption.keySet(),
-                gt -> consumption.getCount(gt) != 0,
-                gt -> new AbstractGoods(gt, consumption.getCount(gt)));
+                        gt -> consumption.getCount(gt) != 0,
+                        gt -> new AbstractGoods(gt, consumption.getCount(gt)));
     }
 
     /**
@@ -660,7 +671,6 @@ public final class UnitType extends BuildableType implements Consumer {
         return getModifiers(id);
     }
 
-
     // Override FreeColObject
 
     /**
@@ -669,7 +679,8 @@ public final class UnitType extends BuildableType implements Consumer {
     @Override
     public <T extends FreeColObject> boolean copyIn(T other) {
         UnitType o = copyInCast(other, UnitType.class);
-        if (o == null || !super.copyIn(o)) return false;
+        if (o == null || !super.copyIn(o))
+            return false;
         this.baseOffence = o.getBaseOffence();
         this.baseDefence = o.getBaseDefence();
         this.space = o.getSpace();
@@ -692,7 +703,6 @@ public final class UnitType extends BuildableType implements Consumer {
         return true;
     }
 
-
     // Serialization
 
     private static final String CONSUMES_TAG = "consumes";
@@ -915,8 +925,8 @@ public final class UnitType extends BuildableType implements Consumer {
         price = xr.getAttribute(PRICE_TAG, parent.price);
 
         mercenaryPrice = xr.getAttribute(MERCENARY_PRICE_TAG,
-                                         parent.mercenaryPrice);
-        
+                parent.mercenaryPrice);
+
         expertProduction = xr.getType(spec, EXPERT_PRODUCTION_TAG,
                 GoodsType.class, parent.expertProduction);
 
@@ -945,7 +955,8 @@ public final class UnitType extends BuildableType implements Consumer {
             defaultRole = parent.defaultRole;
 
             if (parent.consumption != null) {
-                if (consumption == null) consumption = new TypeCountMap<>();
+                if (consumption == null)
+                    consumption = new TypeCountMap<>();
                 consumption.putAll(parent.consumption);
             }
 
@@ -968,24 +979,24 @@ public final class UnitType extends BuildableType implements Consumer {
 
         if (CONSUMES_TAG.equals(tag)) {
             addConsumption(xr.getType(spec, ID_ATTRIBUTE_TAG,
-                    GoodsType.class, (GoodsType)null),
+                    GoodsType.class, (GoodsType) null),
                     xr.getAttribute(VALUE_TAG, UNDEFINED));
             xr.closeTag(CONSUMES_TAG);
 
-        // @compat 0.11.0
+            // @compat 0.11.0
         } else if (OLD_DEFAULT_EQUIPMENT_TAG.equals(tag)) {
             xr.swallowTag(OLD_DEFAULT_EQUIPMENT_TAG);
-        // end @compat 0.11.0
+            // end @compat 0.11.0
 
         } else if (DEFAULT_ROLE_TAG.equals(tag)) {
             defaultRole = xr.getType(spec, ID_ATTRIBUTE_TAG,
                     Role.class, spec.getDefaultRole());
             xr.closeTag(DEFAULT_ROLE_TAG);
 
-        // @compat 0.11.6
+            // @compat 0.11.6
         } else if (DOWNGRADE_TAG.equals(tag) || UPGRADE_TAG.equals(tag)) {
             xr.closeTag(tag, Scope.TAG);
-        // end @compat 0.11.6
+            // end @compat 0.11.6
 
         } else {
             super.readChild(xr);
@@ -995,8 +1006,9 @@ public final class UnitType extends BuildableType implements Consumer {
     /**
      * {@inheritDoc}
      */
-    public String getXMLTagName() { return TAG; }
-
+    public String getXMLTagName() {
+        return TAG;
+    }
 
     // Override Object
 
