@@ -456,10 +456,14 @@ public class ColonyPlan {
             } else if (g.getMilitary()) {
                 militaryGoodsTypes.add(g);
             } else if (g.isRawBuildingMaterial()) {
-                rawBuildingGoodsTypes.add(g);
+                if (g.isRawMaterialForUnstorableBuildingMaterial()) {
+                    rawBuildingGoodsTypes.add(g);
+                }
             } else if (g.isBuildingMaterial()
-                && g.getInputType().isRawBuildingMaterial()) {
-                buildingGoodsTypes.add(g);
+                    && g.getInputType().isRawBuildingMaterial()) {
+                if (!g.isStorable()) {
+                    buildingGoodsTypes.add(g);
+                }
             } else if (g.isNewWorldGoodsType()) {
                 rawLuxuryGoodsTypes.add(g);
             } else if (g.isRefined()
@@ -639,13 +643,13 @@ public class ColonyPlan {
             if ("conquest".equals(advantage)) factor = 1.2;
             ret = prioritize(type, MILITARY_WEIGHT * factor,
                 1.0/*FIXME: amount present wrt amount to equip*/);
-        } else if (goodsType.isBuildingMaterial()) {
+        } else if (goodsType.isBuildingMaterial() && !goodsType.isStorable()) {
             ret = prioritize(type, BUILDING_WEIGHT * factor,
                 1.0/*FIXME: need for this type*/);
         } else if (goodsType.isLibertyType()) {
             if (player.isREF()) return false; // no bells for REF colonies
             ret = prioritize(type, LIBERTY_WEIGHT,
-                (colony.getSoL() >= 100) ? 0.01 : 1.0);
+                (colony.getSonsOfLiberty() >= 100) ? 0.01 : 1.0);
         } else if (goodsType.isImmigrationType()) {
             if ("immigration".equals(advantage)) factor = 1.2;
             ret = prioritize(type, IMMIGRATION_WEIGHT * factor,
@@ -709,8 +713,7 @@ public class ColonyPlan {
                     && colony.getTile().isShore()) {
                     int landFood = 0, seaFood = 0;
                     for (Tile t : transform(colony.getTile().getSurroundingTiles(1,1),
-                            t2 -> (t2.getOwningSettlement() == colony
-                                || player.canClaimForSettlement(t2)))) {
+                            t2 -> (t2.getOwningSettlement() == colony || player.canClaimForSettlement(t2)))) {
                         for (AbstractGoods ag : t.getSortedPotential()) {
                             if (ag.isFoodType()) {
                                 if (t.isLand()) {
@@ -895,7 +898,7 @@ public class ColonyPlan {
                 .reversed();
 
         // If we need liberty put it before the new world production.
-        if (colony.getSoL() < 100) {
+        if (colony.getSonsOfLiberty() < 100) {
             produce.addAll(0, transform(libertyGoodsTypes,
                     gt -> production.containsKey(gt),
                     Function.<GoodsType>identity(), productionComparator));
@@ -1095,13 +1098,8 @@ public class ColonyPlan {
      * @param colony The {@code Colony} storing the equipment.
      * @return True if the unit was equipped.
      */
-    private static boolean fullEquipUnit(Specification spec, Unit unit,
-                                         Role role, Colony colony) {
-        return (role.isOffensive())
-            ? any(transform(spec.getMilitaryRoles(),
-                            r -> unit.roleIsAvailable(r)
-                                && colony.equipForRole(unit, r, r.getMaximumCount())))
-            : colony.equipForRole(unit, role, role.getMaximumCount());
+    private static boolean fullEquipUnit(Specification spec, Unit unit, Role role, Colony colony) {
+        return colony.equipForRole(unit, role, role.getMaximumCount());
     }
 
     /**
@@ -1156,10 +1154,15 @@ public class ColonyPlan {
                 if (workers.size() <= 1) break;
                 Role role = outdoorRole;
                 if (role == null) {
-                    if ((role = u.getMilitaryRole()) == null) continue;
-                }
-                if (u.getType() == role.getExpertUnit()
-                    && fullEquipUnit(spec(), u, role, col)) {
+                    for (Role r : u.getSortedMilitaryRoles()) {
+                        if (u.getType() == r.getExpertUnit() && fullEquipUnit(spec(), u, r, col)) {
+                            workers.remove(u);
+                            lb.add(u.getId(), "(", u.getType().getSuffix(),
+                                ") -> ", r.getSuffix(), "\n");
+                            break;
+                        }
+                    }
+                } else if (u.getType() == role.getExpertUnit() && fullEquipUnit(spec(), u, role, col)) {
                     workers.remove(u);
                     lb.add(u.getId(), "(", u.getType().getSuffix(),
                         ") -> ", role.getSuffix(), "\n");
@@ -1183,11 +1186,17 @@ public class ColonyPlan {
         for (Unit u : sort(workers, soldierComparator)) {
             if (workers.size() <= 1) break;
             if (!col.isBadlyDefended()) break;
-            Role role = u.getMilitaryRole();
-            if (role != null && fullEquipUnit(spec(), u, role, col)) {
-                workers.remove(u);
-                lb.add(u.getId(), "(", u.getType().getSuffix(), ") -> ",
-                       u.getRoleSuffix(), "\n");
+            if (u.getSkillLevel() > 0) {
+                // Stops experts from being used as soldiers
+                continue;
+            }
+            for (Role role : u.getSortedMilitaryRoles()) {
+                if (role != null && fullEquipUnit(spec(), u, role, col)) { 
+                    workers.remove(u);
+                    lb.add(u.getId(), "(", u.getType().getSuffix(), ") -> ",
+                            u.getRoleSuffix(), "\n");
+                    break;
+                }
             }
         }
 
@@ -1435,13 +1444,15 @@ plans:          for (WorkLocationPlan w : getFoodPlans()) {
 
         // Rearm what remains as far as possible.
         for (Unit u : sort(workers, soldierComparator)) {
-            Role role = u.getMilitaryRole();
-            if (role == null) continue;
-            if (fullEquipUnit(spec(), u, role, col)) {
-                lb.add("    ", u.getId(), "(", u.getType().getSuffix(),
-                       ") -> ", u.getRoleSuffix(), "\n");
-                workers.remove(u);
-            } else break;
+            if (u.getSkillLevel() > 0) continue;
+            for (Role role : u.getSortedMilitaryRoles()) {
+                if (fullEquipUnit(spec(), u, role, col)) {
+                    lb.add("    ", u.getId(), "(", u.getType().getSuffix(),
+                           ") -> ", u.getRoleSuffix(), "\n");
+                    workers.remove(u);
+                    break;
+                }
+            }
         }
         for (Unit u : transform(col.getUnits(), u -> !u.hasDefaultRole())) {
             logger.warning("assignWorkers bogus role for " + u);
