@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2015   The FreeCol Team
+ *  Copyright (C) 2002-2022   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -19,6 +19,7 @@
 
 package net.sf.freecol.server.ai.mission;
 
+import java.util.Comparator;
 import java.util.List;
 import java.util.logging.Logger;
 
@@ -36,6 +37,7 @@ import net.sf.freecol.common.model.pathfinding.CostDecider;
 import net.sf.freecol.common.model.pathfinding.CostDeciders;
 import net.sf.freecol.common.model.pathfinding.GoalDecider;
 import net.sf.freecol.common.model.pathfinding.GoalDeciders;
+import static net.sf.freecol.common.util.CollectionUtils.*;
 import net.sf.freecol.common.util.LogBuilder;
 import net.sf.freecol.server.ai.AIMain;
 import net.sf.freecol.server.ai.AIMessage;
@@ -47,10 +49,12 @@ import net.sf.freecol.server.ai.AIUnit;
  * FIXME: acquire protection
  * FIXME: Better avoidance of enemy units
  */
-public class CashInTreasureTrainMission extends Mission {
+public final class CashInTreasureTrainMission extends Mission {
 
     private static final Logger logger = Logger.getLogger(CashInTreasureTrainMission.class.getName());
 
+    public static final String TAG = "cashInTreasureTrainMission";
+
     /** The tag for this mission. */
     private static final String tag = "AI treasureTrain";
 
@@ -62,23 +66,24 @@ public class CashInTreasureTrainMission extends Mission {
 
 
     /**
-     * Creates a mission for the given <code>AIUnit</code>.
+     * Creates a mission for the given {@code AIUnit}.
      *
      * @param aiMain The main AI-object.
-     * @param aiUnit The <code>AIUnit</code> this mission is created for.
-     * @param target The target <code>Location</code> for this mission.
+     * @param aiUnit The {@code AIUnit} this mission is created for.
+     * @param target The target {@code Location} for this mission.
      */
     public CashInTreasureTrainMission(AIMain aiMain, AIUnit aiUnit,
                                       Location target) {
-        super(aiMain, aiUnit, target);
+        super(aiMain, aiUnit);
+
+        setTarget(target);
     }
 
     /**
-     * Creates a new <code>CashInTreasureTrainMission</code> and reads
-     * the given element.
+     * Creates a new {@code CashInTreasureTrainMission} from a reader.
      *
      * @param aiMain The main AI-object.
-     * @param aiUnit The <code>AIUnit</code> this mission is created for.
+     * @param aiUnit The {@code AIUnit} this mission is created for.
      * @param xr The input stream containing the XML.
      * @throws XMLStreamException if a problem was encountered
      *      during parsing.
@@ -94,8 +99,8 @@ public class CashInTreasureTrainMission extends Mission {
     /**
      * Assign a carrier for this treasure.
      *
-     * @param lb A <code>LogBuilder</code> to log to.
-     * @return A suitable carrier <code>AIUnit</code>, to which this unit
+     * @param lb A {@code LogBuilder} to log to.
+     * @return A suitable carrier {@code AIUnit}, to which this unit
      *     has been queued for transport.
      */
     private AIUnit assignCarrier(LogBuilder lb) {
@@ -109,15 +114,9 @@ public class CashInTreasureTrainMission extends Mission {
 
         // Pick the closest carrier and queue this unit.
         final Location here = unit.getLocation();
-        int turns = Unit.MANY_TURNS;
-        Unit closest = null;
-        for (Unit c : carriers) {
-            int t = c.getTurnsToReach(here);
-            if (turns > t) {
-                turns = t;
-                closest = c;
-            }
-        }
+        final Comparator<Unit> comp = cachingIntComparator(u ->
+            u.getTurnsToReach(here));
+        Unit closest = minimize(carriers, comp);
         final AIMain aiMain = getAIMain();
         TransportMission tm;
         AIUnit aiCarrier;
@@ -137,8 +136,8 @@ public class CashInTreasureTrainMission extends Mission {
     /**
      * Extract a valid target for this mission from a path.
      *
-     * @param aiUnit A <code>AIUnit</code> to perform the mission.
-     * @param path A <code>PathNode</code> to extract a target from,
+     * @param aiUnit A {@code AIUnit} to perform the mission.
+     * @param path A {@code PathNode} to extract a target from,
      *     (uses the unit location if null).
      * @return A target for this mission, or null if none found.
      */
@@ -147,9 +146,9 @@ public class CashInTreasureTrainMission extends Mission {
         final Location loc = path.getLastNode().getLocation();
         Colony colony = loc.getColony();
         return (loc instanceof Europe
-            && invalidReason(aiUnit, loc) == null) ? loc
+            && invalidMissionReason(aiUnit, loc) == null) ? loc
             : (colony != null
-                && invalidReason(aiUnit, colony) == null) ? colony
+                && invalidMissionReason(aiUnit, colony) == null) ? colony
             : null;
     }
 
@@ -157,8 +156,8 @@ public class CashInTreasureTrainMission extends Mission {
      * Evaluate a potential cashin mission for a given unit and
      * path.
      *
-     * @param aiUnit The <code>AIUnit</code> to do the mission.
-     * @param path A <code>PathNode</code> to take to the target.
+     * @param aiUnit The {@code AIUnit} to do the mission.
+     * @param path A {@code PathNode} to take to the target.
      * @return A score for the proposed mission.
      */
     public static int scorePath(AIUnit aiUnit, PathNode path) {
@@ -174,10 +173,10 @@ public class CashInTreasureTrainMission extends Mission {
     /**
      * Makes a goal decider that checks cash in sites.
      *
-     * @param aiUnit The <code>AIUnit</code> to search with.
+     * @param aiUnit The {@code AIUnit} to search with.
      * @param deferOK Keep track of the nearest colonies to use as a
      *     fallback destination.
-     * @return A suitable <code>GoalDecider</code>.
+     * @return A suitable {@code GoalDecider}.
      */
     private static GoalDecider getGoalDecider(final AIUnit aiUnit,
                                               final boolean deferOK) {
@@ -214,10 +213,10 @@ public class CashInTreasureTrainMission extends Mission {
     /**
      * Find a suitable cash in location for this unit.
      *
-     * @param aiUnit The <code>AIUnit</code> to execute this mission.
+     * @param aiUnit The {@code AIUnit} to execute this mission.
      * @param range The maximum number of moves to search.
      * @param deferOK Enables deferring to a fallback colony.
-     * @return A <code>PathNode</code> to the target, or null if not found
+     * @return A {@code PathNode} to the target, or null if not found
      *     which includes the case when Europe should be preferred (because
      *     the unit can not get there by itself).
      */
@@ -231,15 +230,14 @@ public class CashInTreasureTrainMission extends Mission {
         final Unit carrier = unit.getCarrier();
         final CostDecider standardCd
             = CostDeciders.avoidSettlementsAndBlockingUnits();
-        PathNode path;
 
-        if (player.getNumberOfSettlements() <= 0 || start == null) {
+        if (!player.hasSettlements() || start == null) {
             // No settlements or not on the map, so go straight to
             // Europe.  If Europe does not exist then this mission is
             // doomed.
             return (europe == null) ? null
                 : unit.findPath(unit.getLocation(), europe, carrier,
-                                standardCd);
+                                standardCd, null);
         }
 
         // Can the unit get to a cash in site?
@@ -250,13 +248,13 @@ public class CashInTreasureTrainMission extends Mission {
     /**
      * Finds a suitable cashin target for the supplied unit.
      *
-     * @param aiUnit The <code>AIUnit</code> to test.
+     * @param aiUnit The {@code AIUnit} to test.
      * @param range The maximum number of moves to search.
      * @param deferOK Enables deferring to a fallback colony.
-     * @return A <code>PathNode</code> to the target, or null if none found.
+     * @return A {@code PathNode} to the target, or null if none found.
      */
-    public static Location findTarget(AIUnit aiUnit, int range, 
-                                      boolean deferOK) {
+    public static Location findMissionTarget(AIUnit aiUnit, int range, 
+                                             boolean deferOK) {
         PathNode path = findTargetPath(aiUnit, range, deferOK);
         return (path != null) ? extractTarget(aiUnit, path)
             : Location.upLoc(findCircleTarget(aiUnit,
@@ -266,11 +264,11 @@ public class CashInTreasureTrainMission extends Mission {
     /**
      * Why would this mission be invalid with the given unit?
      *
-     * @param aiUnit The <code>AIUnit</code> to test.
+     * @param aiUnit The {@code AIUnit} to test.
      * @return A reason why the mission would be invalid with the unit,
      *     or null if none found.
      */
-    private static String invalidMissionReason(AIUnit aiUnit) {
+    private static String invalidUnitReason(AIUnit aiUnit) {
         String reason = invalidAIUnitReason(aiUnit);
         if (reason != null) return reason;
         final Unit unit = aiUnit.getUnit();
@@ -283,8 +281,8 @@ public class CashInTreasureTrainMission extends Mission {
      * Why is this mission invalid with a given colony target, given that
      * intermediate colonies are excluded.
      *
-     * @param aiUnit The <code>AIUnit</code> to check.
-     * @param colony The potential target <code>Colony</code>.
+     * @param aiUnit The {@code AIUnit} to check.
+     * @param colony The potential target {@code Colony}.
      * @return A reason for mission invalidity, or null if none found.
      */
     private static String invalidFullColonyReason(AIUnit aiUnit,
@@ -302,8 +300,8 @@ public class CashInTreasureTrainMission extends Mission {
      * Why is this mission invalid with a given colony target, given that
      * intermediate colonies are included.
      *
-     * @param aiUnit The <code>AIUnit</code> to check.
-     * @param colony The potential target <code>Colony</code>.
+     * @param aiUnit The {@code AIUnit} to check.
+     * @param colony The potential target {@code Colony}.
      * @return A reason for mission invalidity, or null if none found.
      */
     private static String invalidColonyReason(AIUnit aiUnit, Colony colony) {
@@ -313,8 +311,8 @@ public class CashInTreasureTrainMission extends Mission {
     /**
      * Why is this mission invalid with a given Europe target?
      *
-     * @param aiUnit The <code>AIUnit</code> to check.
-     * @param europe The potential target <code>Europe</code>.
+     * @param aiUnit The {@code AIUnit} to check.
+     * @param europe The potential target {@code Europe}.
      * @return A reason for mission invalidity, or null if none found.
      */
     private static String invalidEuropeReason(AIUnit aiUnit, Europe europe) {
@@ -322,13 +320,23 @@ public class CashInTreasureTrainMission extends Mission {
     }
 
     /**
-     * Why would this mission be invalid with the given AI unit and location?
+     * Why would this mission be invalid with the given AI unit?
      *
-     * @param aiUnit The <code>AIUnit</code> to check.
-     * @param loc The <code>Location</code> to check.
+     * @param aiUnit The {@code AIUnit} to test.
      * @return A reason for invalidity, or null if none found.
      */
-    public static String invalidReason(AIUnit aiUnit, Location loc) {
+    public static String invalidMissionReason(AIUnit aiUnit) {
+        return invalidUnitReason(aiUnit);
+    }
+
+    /**
+     * Why would this mission be invalid with the given AI unit and location?
+     *
+     * @param aiUnit The {@code AIUnit} to check.
+     * @param loc The {@code Location} to check.
+     * @return A reason for invalidity, or null if none found.
+     */
+    public static String invalidMissionReason(AIUnit aiUnit, Location loc) {
         String reason = invalidMissionReason(aiUnit);
         return (reason != null)
             ? reason
@@ -339,16 +347,6 @@ public class CashInTreasureTrainMission extends Mission {
             : Mission.TARGETINVALID;
     }
 
-    /**
-     * Why would this mission be invalid with the given AI unit?
-     *
-     * @param aiUnit The <code>AIUnit</code> to test.
-     * @return A reason for invalidity, or null if none found.
-     */
-    public static String invalidReason(AIUnit aiUnit) {
-        return invalidMissionReason(aiUnit);
-    }
-
 
     // Implement Mission
     //   Inherit dispose, getTransportDestination, isOneTime
@@ -385,7 +383,7 @@ public class CashInTreasureTrainMission extends Mission {
      */
     @Override
     public Location findTarget() {
-        return findTarget(getAIUnit(), 20, true);
+        return findMissionTarget(getAIUnit(), 20, true);
     }
 
     /**
@@ -393,7 +391,7 @@ public class CashInTreasureTrainMission extends Mission {
      */
     @Override
     public String invalidReason() {
-        return invalidReason(getAIUnit(), target);
+        return invalidMissionReason(getAIUnit(), target);
     }
 
     /**
@@ -418,10 +416,12 @@ public class CashInTreasureTrainMission extends Mission {
             case MOVE: // Arrived
                 break;
 
-            case MOVE_HIGH_SEAS: case MOVE_NO_MOVES:
-            case MOVE_NO_REPAIR: case MOVE_ILLEGAL:
+            case MOVE_HIGH_SEAS: case MOVE_NO_MOVES: case MOVE_ILLEGAL:
                 return lbWait(lb);
 
+            case MOVE_NO_REPAIR:
+                return lbFail(lb, false, AIUNITDIED);
+
             case MOVE_NO_ACCESS_EMBARK: case MOVE_NO_TILE:
                 return this;
 
@@ -440,8 +440,7 @@ public class CashInTreasureTrainMission extends Mission {
             final Europe europe = getUnit().getOwner().getEurope();
             if (unit.canCashInTreasureTrain()) {
                 AIUnit aiCarrier = null;
-                boolean cashin = unit.isInEurope()
-                    || europe == null
+                boolean cashin = europe == null || unit.isInEurope()
                     || unit.getTransportFee() == 0;
                 if (!cashin && aiUnit.getTransport() == null) {
                     cashin = assignCarrier(lb) == null;
@@ -486,14 +485,5 @@ public class CashInTreasureTrainMission extends Mission {
      * {@inheritDoc}
      */
     @Override
-    public String getXMLTagName() { return getXMLElementTagName(); }
-
-    /**
-     * Gets the tag name of the root element representing this object.
-     *
-     * @return "cashInTreasureTrainMission".
-     */
-    public static String getXMLElementTagName() {
-        return "cashInTreasureTrainMission";
-    }
+    public String getXMLTagName() { return TAG; }
 }
