@@ -456,6 +456,7 @@ public final class ScoutingMission extends Mission {
             return lbFail(lb, false, AIUNITDIED);
 
         case MOVE_NO_ACCESS_EMBARK:
+            setTarget(target);
             return this;
 
         case MOVE_NO_TILE:
@@ -538,7 +539,9 @@ public final class ScoutingMission extends Mission {
     protected void readAttributes(FreeColXMLReader xr) throws XMLStreamException {
         super.readAttributes(xr);
 
-        setTarget(xr.getLocationAttribute(getGame(), TARGET_TAG, false));
+        // Do not use setTarget, yet, it does complex stuff that may not
+        // be initialized
+        this.target = xr.getLocationAttribute(getGame(), TARGET_TAG, false);
     }
 
     /**
