@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2015   The FreeCol Team
+ *  Copyright (C) 2002-2022   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -26,7 +26,6 @@ import java.util.HashMap;
 import java.util.List;
 import java.util.Random;
 import java.util.logging.Logger;
-import java.util.stream.Collectors;
 
 import net.sf.freecol.common.model.Game;
 import net.sf.freecol.common.model.LandMap;
@@ -40,7 +39,6 @@ import net.sf.freecol.common.model.Specification;
 import net.sf.freecol.common.model.Tile;
 import net.sf.freecol.common.model.TileImprovement;
 import net.sf.freecol.common.model.TileImprovementType;
-import net.sf.freecol.common.model.TileItemContainer;
 import net.sf.freecol.common.model.TileType;
 import net.sf.freecol.common.option.MapGeneratorOptions;
 import net.sf.freecol.common.option.OptionGroup;
@@ -52,7 +50,7 @@ import net.sf.freecol.server.model.ServerRegion;
 
 
 /**
- * Class for making a <code>Map</code> based upon a land map.
+ * Class for making a {@code Map} based upon a land map.
  *
  * FIXME: dynamic lakes, mountains and hills
  */
@@ -64,26 +62,11 @@ public class TerrainGenerator {
     public static final int LAND_REGION_MIN_SCORE = 5;
     public static final int LAND_REGION_MAX_SIZE = 75;
 
-    /** The Game to generate for. */
-    private final Game game;
-
-    /** The game to selectively import from. */
-    private final Game importGame;
-
-    /**
-     * The pseudo-random number source to use.
-     *
-     * Uses of the PRNG are usually logged in FreeCol, but the use is
-     * so intense here and this code is called pre-game, so we
-     * intentionally skip the logging here.
-     */
+    /** The pseudo random number generator. */
     private final Random random;
 
-    /** The cached map generator options. */
-    private final OptionGroup mapOptions;
-
-    /** The cache of the Specification. */
-    private final Specification spec;
+    /** A cached random integer source. */
+    private final RandomIntCache cache;
 
     /** The cached land and ocean tile types. */
     private List<TileType> landTileTypes = null;
@@ -91,19 +74,17 @@ public class TerrainGenerator {
 
 
     /**
-     * Creates a new <code>TerrainGenerator</code>.
+     * Creates a new {@code TerrainGenerator}.
+     * 
+     * FIXME: cache or randomizer???
      *
-     * @param game The <code>Game</code> to generate for.
-     * @param importGame A <code>Game</code> to selectively import from.
-     * @param random A <code>Random</code> number source.
-     * @see #createMap
+     * @param random A pseudo-random number source.
+     * @see #generateMap
      */
-    public TerrainGenerator(Game game, Game importGame, Random random) {
-        this.game = game;
-        this.importGame = importGame;
+    public TerrainGenerator(Random random) {
         this.random = random;
-        this.mapOptions = game.getMapGeneratorOptions();
-        this.spec = game.getSpecification();
+        this.cache = new RandomIntCache(logger, "terrain", random,
+                                        1 << 16, 512);
     }
 
 
@@ -117,65 +98,53 @@ public class TerrainGenerator {
     /**
      * Gets the approximate number of land tiles.
      *
+     * @param game The {@code Game} to generate for.
      * @return The approximate number of land tiles
      */
-    private int getApproximateLandCount() {
+    private int getApproximateLandCount(Game game) {
+        final OptionGroup mapOptions = game.getMapGeneratorOptions();
         return mapOptions.getInteger(MapGeneratorOptions.MAP_WIDTH)
             * mapOptions.getInteger(MapGeneratorOptions.MAP_HEIGHT)
             * mapOptions.getInteger(MapGeneratorOptions.LAND_MASS)
             / 100;
     }
 
-    /**
-     * Creates a random tile for the specified position.
-     *
-     * @param x The tile x coordinate.
-     * @param y The tile y coordinate.
-     * @param landMap A boolean array defining where the land is.
-     * @param latitude The tile latitude.
-     * @return The created tile.
-     */
-    private Tile createTile(int x, int y, LandMap landMap, int latitude) {
-        return (landMap.isLand(x, y))
-            ? new Tile(game, getRandomLandTileType(latitude), x, y)
-            : new Tile(game, getRandomOceanTileType(latitude), x, y);
-    }
-
     /**
      * Gets a random land tile type based on the latitude.
      *
+     * @param game The {@code Game} to generate for.
      * @param latitude The location of the tile relative to the north/south
      *     poles and equator:
      *     0 is the mid-section of the map (equator)
      *     +/-90 is on the bottom/top of the map (poles).
      * @return A suitable random land tile type.
      */
-    private TileType getRandomLandTileType(int latitude) {
+    private TileType getRandomLandTileType(Game game, int latitude) {
+        final Specification spec = game.getSpecification();
         if (landTileTypes == null) {
             // Do not generate elevated and water tiles at this time
             // they are created elsewhere.
-            landTileTypes = spec.getTileTypeList().stream()
-                .filter(t -> !t.isElevation() && !t.isWater())
-                .collect(Collectors.toList());
+            landTileTypes = transform(spec.getTileTypeList(),
+                                      t -> !t.isElevation() && !t.isWater());
         }
-        return getRandomTileType(landTileTypes, latitude);
+        return getRandomTileType(game, landTileTypes, latitude);
     }
 
     /**
      * Gets a random ocean tile type.
      *
+     * @param game The {@code Game} to generate for.
      * @param latitude The latitude of the proposed tile.
      * @return A suitable random ocean tile type.
      */
-    private TileType getRandomOceanTileType(int latitude) {
+    private TileType getRandomOceanTileType(Game game, int latitude) {
+        final Specification spec = game.getSpecification();
         if (oceanTileTypes == null) {
-            oceanTileTypes = spec.getTileTypeList().stream()
-                .filter(t -> t.isWater()
-                    && t.isHighSeasConnected()
-                    && !t.isDirectlyHighSeasConnected())
-                .collect(Collectors.toList());
+            oceanTileTypes = transform(spec.getTileTypeList(),
+                                       t -> t.isWater() && t.isHighSeasConnected()
+                                           && !t.isDirectlyHighSeasConnected());
         }
-        return getRandomTileType(oceanTileTypes, latitude);
+        return getRandomTileType(game, oceanTileTypes, latitude);
     }
 
     /**
@@ -183,18 +152,21 @@ public class TerrainGenerator {
      *
      * FIXME: Can be used for mountains and rivers too.
      *
-     * @param candidates A list of <code>TileType</code>s to use for
+     * @param game The {@code Game} to generate for.
+     * @param candidates A list of {@code TileType}s to use for
      *     calculations.
      * @param latitude The tile latitude.
-     * @return A suitable <code>TileType</code>.
+     * @return A suitable {@code TileType}.
      */
-    private TileType getRandomTileType(List<TileType> candidates,
+    private TileType getRandomTileType(Game game, List<TileType> candidates,
                                        int latitude) {
+        final OptionGroup mapOptions = game.getMapGeneratorOptions();
+        final Specification spec = game.getSpecification();
         // decode options
         final int forestChance
-            = mapOptions.getInteger(MapGeneratorOptions.FOREST_NUMBER);
+            = mapOptions.getRange(MapGeneratorOptions.FOREST_NUMBER);
         final int temperaturePreference
-            = mapOptions.getInteger(MapGeneratorOptions.TEMPERATURE);
+            = mapOptions.getRange(MapGeneratorOptions.TEMPERATURE);
 
         // temperature calculation
         int poleTemperature = -20;
@@ -224,22 +196,18 @@ public class TerrainGenerator {
             break;
         }
 
-        final Specification spec = game.getSpecification();
         int temperatureRange = equatorTemperature-poleTemperature;
         int localeTemperature = poleTemperature + (90 - Math.abs(latitude))
             * temperatureRange/90;
         int temperatureDeviation = 7; // +/- 7 degrees randomization
-        localeTemperature += randomInt(logger, "Temperature", random,
-                                       temperatureDeviation * 2)
+        localeTemperature += this.cache.nextInt(temperatureDeviation * 2)
             - temperatureDeviation;
         localeTemperature = limitToRange(localeTemperature, -20, 40);
 
         // humidity calculation
-        int localeHumidity = spec.getRangeOption(MapGeneratorOptions.HUMIDITY)
-            .getValue();
+        int localeHumidity = spec.getRange(MapGeneratorOptions.HUMIDITY);
         int humidityDeviation = 20; // +/- 20% randomization
-        localeHumidity += randomInt(logger, "Humidity", random,
-                                    humidityDeviation * 2)
+        localeHumidity += this.cache.nextInt(humidityDeviation * 2)
             - humidityDeviation;
         localeHumidity = limitToRange(localeHumidity, 0, 100);
 
@@ -263,7 +231,7 @@ public class TerrainGenerator {
             throw new RuntimeException("No TileType for"
                 + " temperature==" + localeTemperature);
         case 1:
-            return candidateTileTypes.get(0);
+            return first(candidateTileTypes);
         default:
             break;
         }
@@ -286,13 +254,14 @@ public class TerrainGenerator {
             throw new RuntimeException("No TileType for"
                 + " humidity==" + localeHumidity);
         case 1:
-            return candidateTileTypes.get(0);
+            return first(candidateTileTypes);
         default:
             break;
         }
 
         // Filter the candidates by forest presence.
-        boolean forested = randomInt(logger, "Forest", random, 100) < forestChance;
+        boolean forested = this.cache.nextInt(100) < forestChance
+            && any(candidateTileTypes, TileType::isForested);
         i = 0;
         while (i < candidateTileTypes.size()) {
             TileType type = candidateTileTypes.get(i);
@@ -309,10 +278,9 @@ public class TerrainGenerator {
             throw new RuntimeException("No TileType for"
                 + " forested==" + forested);
         case 1:
-            return candidateTileTypes.get(0);
+            return first(candidateTileTypes);
         default:
-            return candidateTileTypes.get(randomInt(logger, "Forest tile",
-                                                    random, i));
+            return candidateTileTypes.get(this.cache.nextInt(i));
         }
     }
 
@@ -323,17 +291,18 @@ public class TerrainGenerator {
      * Creates land map regions in the given Map.
      *
      * First, the arctic/antarctic regions are defined, based on
-     * <code>Map.POLAR_HEIGHT</code>.
+     * {@code Map.POLAR_HEIGHT}.
      *
      * For the remaining land tiles, one region per contiguous
      * landmass is created.
      *
-     * @param map The <code>Map</code> to work on.
-     * @param lb A <code>LogBuilder</code> to log to.
-     * @return A list of created <code>ServerRegion</code>s.
+     * @param map The {@code Map} to work on.
+     * @param lb A {@code LogBuilder} to log to.
+     * @return A list of created {@code ServerRegion}s.
      */
     private List<ServerRegion> createLandRegions(Map map, LogBuilder lb) {
         // Create "explorable" land regions
+        final Game game = map.getGame();
         int continents = 0;
         boolean[][] landmap = new boolean[map.getWidth()][map.getHeight()];
         int[][] continentmap = new int[map.getWidth()][map.getHeight()];
@@ -361,7 +330,7 @@ public class TerrainGenerator {
             for (int x = 0; x < map.getWidth(); x++) {
                 if (landmap[x][y]) { // Found a new region.
                     continents++;
-                    boolean[][] continent = Map.floodFill(landmap, x, y);
+                    boolean[][] continent = Map.floodFillBool(landmap, x, y);
 
                     for (int yy = 0; yy < map.getHeight(); yy++) {
                         for (int xx = 0; xx < map.getWidth(); xx++) {
@@ -410,7 +379,7 @@ public class TerrainGenerator {
                         targetsize = continentsize[c]/2;
                     }
                     continents++; //index of the new region in continentmap[][]
-                    boolean[][] newregion = Map.floodFill(splitcontinent,
+                    boolean[][] newregion = Map.floodFillBool(splitcontinent,
                         splitX, splitY, targetsize);
                     for (int x = 0; x < map.getWidth(); x++) {
                         for (int y = 0; y < map.getHeight(); y++) {
@@ -464,44 +433,18 @@ public class TerrainGenerator {
         return Arrays.asList(Arrays.copyOfRange(landregions, 1, continents+1));
     }
 
-    /**
-     * Pick a good tile to put elevated terrain on.
-     *
-     * @param map A <code>Map</code> to choose from.
-     * @return A suitable <code>Tile</code>, or null if none found.
-     */
-    private Tile getGoodMountainTile(Map map) {
-        final TileType hills = spec.getTileType("model.tile.hills");
-        final TileType mountains = spec.getTileType("model.tile.mountains");
-        Tile tile = null;
-        while ((tile = map.getRandomLandTile(random)) != null) {
-            // Can not be high ground already
-            if (tile.getType() != hills && tile.getType() != mountains
-                
-                // Not too close to a mountain range as this would
-                // defeat the purpose of adding random hills
-                && none(tile.getSurroundingTiles(1, 3), t -> t.getType() == mountains)
-
-                // Do not add hills too close to the ocean/lake, as
-                // this helps with good locations for building
-                // colonies on shore.
-                && none(tile.getSurroundingTiles(1, 1), t -> !t.isLand())) {
-                return tile;
-            }
-        }
-        return null;
-    }
-
     /**
      * Creates mountain ranges on the given map.  The number and size
      * of mountain ranges depends on the map size.
      *
      * @param map The map to use.
-     * @param lb A <code>LogBuilder</code> to log to.
-     * @return A list of created <code>ServerRegion</code>s.
+     * @param lb A {@code LogBuilder} to log to.
+     * @return A list of created {@code ServerRegion}s.
      */
     private List<ServerRegion> createMountains(Map map, LogBuilder lb) {
-        List<ServerRegion> result = new ArrayList<>();
+        final Game game = map.getGame();
+        final Specification spec = game.getSpecification();
+        final OptionGroup mapOptions = game.getMapGeneratorOptions();
         float randomHillsRatio = 0.5f;
         // 50% of user settings will be allocated for random hills
         // here and there the rest will be allocated for large
@@ -509,52 +452,62 @@ public class TerrainGenerator {
         int maximumLength
             = Math.max(mapOptions.getInteger(MapGeneratorOptions.MAP_WIDTH),
                 mapOptions.getInteger(MapGeneratorOptions.MAP_HEIGHT)) / 10;
-        int number = (int)((1.0f - randomHillsRatio)
-            * (getApproximateLandCount()
-                / mapOptions.getInteger(MapGeneratorOptions.MOUNTAIN_NUMBER)));
+        int number = Math.round((1.0f - randomHillsRatio)
+            * ((float)getApproximateLandCount(game)
+                / mapOptions.getRange(MapGeneratorOptions.MOUNTAIN_NUMBER)));
         lb.add("Number of mountain tiles is ", number, "\n",
             "Maximum length of mountain ranges is ", maximumLength, "\n");
+        List<ServerRegion> result = new ArrayList<>(number);
 
         // lookup the resources from specification
         final TileType hills = spec.getTileType("model.tile.hills");
         final TileType mountains = spec.getTileType("model.tile.mountains");
         if (hills == null || mountains == null) {
-            throw new RuntimeException("Both Hills and Mountains TileTypes must be defined");
+            throw new RuntimeException("Both Hills and Mountains TileTypes must be defined: " + spec);
         }
 
         // Generate the mountain ranges
-        int counter = 0;
-        for (int tries = 0; tries < 100; tries++) {
-            Tile startTile = getGoodMountainTile(map);
-            if (startTile == null) break;
+        List<Tile> tiles = new ArrayList<>();
+        map.forEachTile(t -> t.isGoodMountainTile(mountains),
+                        t -> tiles.add(t));
+        randomShuffle(logger, "Randomize mountain tiles", tiles, this.random);
 
+        int counter = 0;
+        for (Tile startTile : tiles) {
+            // isGoodMountainTile can change when new mountains are added
+            if (!startTile.isGoodMountainTile(mountains)) continue;
+            
             ServerRegion mountainRegion
                 = new ServerRegion(game, RegionType.MOUNTAIN);
-            startTile.setType(mountains);
-            mountainRegion.addTile(startTile);
             Direction direction = Direction.getRandomDirection("getLand",
-                logger, random);
-            int length = maximumLength
-                - randomInt(logger, "MLen", random, maximumLength/2);
+                logger, this.random);
+            int length = maximumLength - this.cache.nextInt(maximumLength/2);
+            Tile tile = startTile;
             for (int index = 0; index < length; index++) {
-                Tile nextTile = startTile.getNeighbourOrNull(direction);
-                if (nextTile == null || !nextTile.isLand()) continue;
-                nextTile.setType(mountains);
-                mountainRegion.addTile(nextTile);
-                counter++;
-                for (Tile neighbour : nextTile.getSurroundingTiles(1)) {
-                    if (!neighbour.isLand()
-                        || neighbour.getType() == mountains) continue;
-                    int r = randomInt(logger, "MSiz", random, 8);
-                    if (r == 0) {
-                        neighbour.setType(mountains);
-                        mountainRegion.addTile(neighbour);
+                // Raise current tile up as mountain
+                if (tile.getType() != mountains) {
+                    tile.setType(mountains);
+                    mountainRegion.addTile(tile);
+                    counter++;
+                }
+                // Add nothing (1/8), mountains (2/8) or hills (5/8)
+                // to surrounding tiles if suitable and not already a
+                // mountain
+                for (Tile t : tile.getSurroundingTiles(1)) {
+                    if (!t.isGoodHillTile()
+                        || t.getType() == mountains) continue;
+                    int r = this.cache.nextInt(8);
+                    if (r < 2) {
+                        t.setType(mountains);
+                        mountainRegion.addTile(t);
                         counter++;
-                    } else if (r > 2) {
-                        neighbour.setType(hills);
-                        mountainRegion.addTile(neighbour);
+                    } else if (r < 7) {
+                        t.setType(hills);
+                        mountainRegion.addTile(t);
                     }
                 }
+                tile = tile.getNeighbourOrNull(direction);
+                if (tile == null || !tile.isLand()) break;
             }
             int scoreValue = 2 * mountainRegion.getSize();
             mountainRegion.setScoreValue(scoreValue);
@@ -568,15 +521,16 @@ public class TerrainGenerator {
         lb.add("Added ", counter, " mountain range tiles.\n");
 
         // and sprinkle a few random hills/mountains here and there
-        number = (int) (getApproximateLandCount() * randomHillsRatio)
-            / mapOptions.getInteger(MapGeneratorOptions.MOUNTAIN_NUMBER);
-        counter = 0;
-        for (int tries = 0; tries < 1000; tries++) {
-            Tile t = getGoodMountainTile(map);
-            if (t == null) break;
+        tiles.clear();
+        map.forEachTile(Tile::isGoodHillTile, t -> tiles.add(t));
+        randomShuffle(logger, "Randomize hill tiles", tiles, this.random);
 
+        number = (int) (getApproximateLandCount(game) * randomHillsRatio)
+            / mapOptions.getRange(MapGeneratorOptions.MOUNTAIN_NUMBER);
+        counter = 0;
+        for (Tile t : tiles) {
             // 25% mountains, 75% hills
-            boolean m = randomInt(logger, "MorH", random, 4) == 0;
+            boolean m = this.cache.nextInt(4) == 0;
             t.setType((m) ? mountains : hills);
             if (++counter >= number) break;
         }
@@ -588,57 +542,53 @@ public class TerrainGenerator {
      * Creates rivers on the given map. The number of rivers depends
      * on the map size.
      *
-     * @param map The <code>Map</code> to create rivers on.
-     * @param lb A <code>LogBuilder</code> to log to.
-     * @return A list of created <code>ServerRegion</code>s.
+     * @param map The {@code Map} to create rivers on.
+     * @param lb A {@code LogBuilder} to log to.
+     * @return A list of created {@code ServerRegion}s.
      */
     private List<ServerRegion> createRivers(Map map, LogBuilder lb) {
+        final Game game = map.getGame();
+        final Specification spec = game.getSpecification();
+        final OptionGroup mapOptions = game.getMapGeneratorOptions();
         List<ServerRegion> result = new ArrayList<>();
         final TileImprovementType riverType
             = spec.getTileImprovementType("model.improvement.river");
-        final int number = getApproximateLandCount()
-            / mapOptions.getInteger(MapGeneratorOptions.RIVER_NUMBER);
-        int counter = 0;
+        final int number = getApproximateLandCount(game)
+            / mapOptions.getRange(MapGeneratorOptions.RIVER_NUMBER);
         HashMap<Tile, River> riverMap = new HashMap<>();
         List<River> rivers = new ArrayList<>();
 
-        outer: for (int i = 0; i < number; i++) {
-            for (int tries = 0; tries < 100; tries++) {
-                Tile tile = map.getRandomLandTile(random);
-                if (tile == null) break outer;
+        List<Tile> tiles = new ArrayList<>();
+        map.forEachTile(t -> t.isGoodRiverTile(riverType),
+                        t -> tiles.add(t));
+        randomShuffle(logger, "Randomize river tiles", tiles, this.random);
+        
+        int counter = 0;
+        for (Tile tile : tiles) {
+            // Any river here yet?
+            if (riverMap.get(tile) != null)
+                continue;
 
-                if (!riverType.isTileTypeAllowed(tile.getType())) continue;
-
-                // check the river source/spring is not too close to the ocean
-                if (!all(tile.getSurroundingTiles(1, 2),
-                        Tile::isLand)) continue;
-
-                if (riverMap.get(tile) == null) {
-                    // no river here yet
-                    ServerRegion riverRegion = new ServerRegion(game, RegionType.RIVER);
-                    River river = new River(map, riverMap, riverRegion, random);
-                    if (river.flowFromSource(tile)) {
-                        lb.add("Created new river with length ",
-                            river.getLength(), "\n");
-                        result.add(riverRegion);
-                        rivers.add(river);
-                        if (++counter >= number) break;
-                    } else {
-                        lb.add("Failed to generate river.\n");
-                    }
-                    break;
-                }
+            ServerRegion riverRegion = new ServerRegion(game, RegionType.RIVER);
+            River river = new River(map, riverMap, riverRegion, this.random);
+            if (river.flowFromSource(tile)) {
+                lb.add("Created new river with length ",
+                    river.getLength(), "\n");
+                result.add(riverRegion);
+                rivers.add(river);
+                if (++counter >= number) break;
+            } else {
+                lb.add("Failed to generate river at " + tile + ".\n");
             }
         }
         lb.add("Created ", counter, " rivers of maximum ", number, "\n");
 
         for (River river : rivers) {
             ServerRegion region = river.getRegion();
-            int scoreValue = 2 * river.getSections().stream()
-                .mapToInt(rs -> rs.getSize()).sum();
-            region.setScoreValue(scoreValue);
+            int score = 2 * sum(river.getSections(), RiverSection::getSize);
+            region.setScoreValue(score);
             lb.add("Created river region (length ", river.getLength(),
-                ", score value ", scoreValue, ").\n");
+                ", score value ", score, ").\n");
         }
         return result;
     }
@@ -646,13 +596,11 @@ public class TerrainGenerator {
     /**
      * Finds all the lake regions.
      *
-     * @param map The <code>Map</code> to work on.
-     * @param lb A <code>LogBuilder</code> to log to.
-     * @return A list of created <code>ServerRegion</code>s.
+     * @param map The {@code Map} to work on.
+     * @param lb A {@code LogBuilder} to log to.
+     * @return A list of created {@code ServerRegion}s.
      */
     private List<ServerRegion> createLakeRegions(Map map, LogBuilder lb) {
-        final TileType lakeType = spec.getTileType("model.tile.lake");
-
         // Create the water map, and find any tiles that are water but
         // not part of any region (such as the oceans).  These are
         // lake tiles.
@@ -677,23 +625,19 @@ public class TerrainGenerator {
     /**
      * Make lake regions from unassigned lake tiles.
      *
-     * // @compat 0.10.x
-     * // Public/static to allow Map to remediate missing lake regions
-     * // end @compat 0.10.x
-     *
-     * @param map The <code>Map</code> to add to.
-     * @param lakes A list of lake <code>Tile</code>s.
-     * @return A list of new <code>ServerRegion</code>s.
+     * @param map The {@code Map} to add to.
+     * @param lakes A list of lake {@code Tile}s.
+     * @return A list of new {@code ServerRegion}s.
      */
-    public static List<ServerRegion> makeLakes(Map map, List<Tile> lakes) {
+    private List<ServerRegion> makeLakes(Map map, List<Tile> lakes) {
         Game game = map.getGame();
         final TileType lakeType = map.getSpecification()
             .getTileType("model.tile.lake");
-        List<Tile> todo = new ArrayList<>();
-        List<ServerRegion> result = new ArrayList<>();
+        List<Tile> todo = new ArrayList<>(lakes.size()/10);
+        List<ServerRegion> result = new ArrayList<>(lakes.size()/10);
         int lakeCount = 0;
         while (!lakes.isEmpty()) {
-            Tile tile = lakes.get(0);
+            Tile tile = first(lakes);
             if (tile.getRegion() != null) continue;
 
             ServerRegion lakeRegion = new ServerRegion(game, RegionType.LAKE);
@@ -724,22 +668,23 @@ public class TerrainGenerator {
 
     /**
      * Adds a terrain bonus with a probability determined by the
-     * <code>MapGeneratorOptions</code>.
+     * {@code MapGeneratorOptions}.
      *
-     * @param t The <code>Tile</code> to add bonuses to.
+     * @param t The {@code Tile} to add bonuses to.
      * @param generateBonus Generate the bonus or not.
      */
     private void perhapsAddBonus(Tile t, boolean generateBonus) {
-        final Specification spec = t.getSpecification();
+        final Game game = t.getGame();
+        final OptionGroup mapOptions = game.getMapGeneratorOptions();
+        final Specification spec = game.getSpecification();
         TileImprovementType fishBonusLandType
             = spec.getTileImprovementType("model.improvement.fishBonusLand");
         TileImprovementType fishBonusRiverType
             = spec.getTileImprovementType("model.improvement.fishBonusRiver");
         final int bonusNumber
-            = mapOptions.getInteger(MapGeneratorOptions.BONUS_NUMBER);
+            = mapOptions.getRange(MapGeneratorOptions.BONUS_NUMBER);
         if (t.isLand()) {
-            if (generateBonus
-                && randomInt(logger, "Land Resource", random, 100) < bonusNumber) {
+            if (generateBonus && this.cache.nextInt(100) < bonusNumber) {
                 // Create random Bonus Resource
                 t.addResource(createResource(t));
             }
@@ -759,25 +704,24 @@ public class TerrainGenerator {
             // In Col1, ocean tiles with less than 3 land neighbours
             // produce 2 fish, all others produce 4 fish
             if (adjacentLand > 2) {
-                t.add(new TileImprovement(game, t, fishBonusLandType));
+                t.add(new TileImprovement(game, t, fishBonusLandType, null));
             }
 
             // In Col1, the ocean tile in front of a river mouth would
             // get an additional +1 bonus
             // FIXME: This probably has some false positives, means
             // river tiles that are NOT a river mouth next to this tile!
-            if (!t.hasRiver() && adjacentRiver) {
-                t.add(new TileImprovement(game, t, fishBonusRiverType));
+            if (adjacentRiver && !t.hasRiver()) {
+                t.add(new TileImprovement(game, t, fishBonusRiverType, null));
             }
 
             if (t.getType().isHighSeasConnected()) {
                 if (generateBonus && adjacentLand > 1
-                    && randomInt(logger, "Sea resource", random,
-                                 10 - adjacentLand) == 0) {
+                    && this.cache.nextInt(10 - adjacentLand) == 0) {
                     t.addResource(createResource(t));
                 }
             } else {
-                if (randomInt(logger, "Water resource", random, 100) < bonusNumber) {
+                if (this.cache.nextInt(100) < bonusNumber) {
                     // Create random Bonus Resource
                     t.addResource(createResource(t));
                 }
@@ -788,32 +732,27 @@ public class TerrainGenerator {
     /**
      * Create a random resource on a tile.
      *
-     * @param tile The <code>Tile</code> to create the resource on.
+     * @param tile The {@code Tile} to create the resource on.
      * @return The created resource, or null if it is not possible.
      */
     private Resource createResource(Tile tile) {
         if (tile == null) return null;
         ResourceType resourceType = RandomChoice.getWeightedRandom(null, null,
-            tile.getType().getWeightedResources(), random);
+            tile.getType().getResourceTypes(), this.random);
         if (resourceType == null) return null;
         int minValue = resourceType.getMinValue();
         int maxValue = resourceType.getMaxValue();
         int quantity = (minValue == maxValue) ? maxValue
-            : (minValue + randomInt(logger, "Rsiz", random, 
-                                    maxValue - minValue + 1));
-        return new Resource(game, tile, resourceType, quantity);
+            : (minValue + this.cache.nextInt(maxValue - minValue + 1));
+        return new Resource(tile.getGame(), tile, resourceType, quantity);
     }
 
-    // @compat 0.10.x
-    // Only still public to allow reencoding tile styles when loading
-    // pre-save-v-12 savegames.
-    // end @compat 0.10.x
     /**
      * Sets the style of the tiles.
      * Only relevant to water tiles for now.
      * Public because it is used in the river generator.
      *
-     * @param tile The <code>Tile</code> to set the style of.
+     * @param tile The {@code Tile} to set the style of.
      */
     public static void encodeStyle(Tile tile) {
         EnumMap<Direction, Boolean> connections
@@ -852,22 +791,29 @@ public class TerrainGenerator {
     // Main functionality, create the map.
 
     /**
-     * Creates a <code>Map</code>.
+     * Make a {@code Map}.
      *
-     * @param landMap The <code>LandMap</code> to use as a template.
-     * @param lb A <code>LogBuilder</code> to log to.
-     * @return The new <code>Map</code>.
+     * @param game The {@code Game} to generate a map for.
+     * @param importMap An optional {@code Map} to import.
+     * @param landMap The {@code LandMap} to use as a template.
+     * @param lb A {@code LogBuilder} to log to.
+     * @return The new {@code Map}.
      */
-    public Map createMap(LandMap landMap, LogBuilder lb) {
+    public Map generateMap(Game game, Map importMap, LandMap landMap,
+                           LogBuilder lb) {
+        final OptionGroup mapOptions = game.getMapGeneratorOptions();
         final int width = landMap.getWidth();
         final int height = landMap.getHeight();
-        final boolean importTerrain = (importGame != null)
-            && mapOptions.getBoolean(MapGeneratorOptions.IMPORT_TERRAIN);
-        final boolean importBonuses = (importGame != null)
+        final boolean importBonuses = (importMap != null)
             && mapOptions.getBoolean(MapGeneratorOptions.IMPORT_BONUSES);
+        final boolean importRumours = (importMap != null)
+            && mapOptions.getBoolean(MapGeneratorOptions.IMPORT_RUMOURS);
+        final boolean importTerrain = (importMap != null)
+            && mapOptions.getBoolean(MapGeneratorOptions.IMPORT_TERRAIN);
 
-        boolean mapHasLand = false;
         Map map = new Map(game, width, height);
+        game.changeMap(map);
+        
         int minimumLatitude = mapOptions
             .getInteger(MapGeneratorOptions.MINIMUM_LATITUDE);
         int maximumLatitude = mapOptions
@@ -881,13 +827,13 @@ public class TerrainGenerator {
         java.util.Map<String, ServerRegion> regionMap = new HashMap<>();
         if (importTerrain) { // Import the regions
             lb.add("Imported regions: ");
-            for (Region r : importGame.getMap().getRegions()) {
+            for (Region r : importMap.getRegions()) {
                 ServerRegion region = new ServerRegion(game, r);
                 map.addRegion(region);
                 regionMap.put(r.getId(), region);
                 lb.add(" ", region.toString());
             }
-            for (Region r : importGame.getMap().getRegions()) {
+            for (Region r : importMap.getRegions()) {
                 ServerRegion region = regionMap.get(r.getId());
                 Region x = r.getParent();
                 if (x != null) x = regionMap.get(x.getId());
@@ -900,32 +846,18 @@ public class TerrainGenerator {
             lb.add("\n");
         }
 
+        final Map.Layer layer = (importRumours) ? Map.Layer.RUMOURS
+            : (importBonuses) ? Map.Layer.RESOURCES
+            : Map.Layer.RIVERS;
         List<Tile> fixRegions = new ArrayList<>();
-        for (int y = 0; y < height; y++) {
-            int latitude = map.getLatitude(y);
-            for (int x = 0; x < width; x++) {
-                if (landMap.isLand(x, y)) mapHasLand = true;
-                Tile t, importTile = null;
+        map.populateTiles((x, y) -> {
+                Tile t, otherTile = null;
                 if (importTerrain
-                    && importGame.getMap().isValid(x, y)
-                    && (importTile = importGame.getMap().getTile(x, y)) != null
-                    && importTile.isLand() == landMap.isLand(x, y)) {
-                    String id = importTile.getType().getId();
-                    t = new Tile(game, spec.getTileType(id), x, y);
-                    if (importTile.getMoveToEurope() != null) {
-                        t.setMoveToEurope(importTile.getMoveToEurope());
-                    }
-                    if (importTile.getTileItemContainer() != null) {
-                        TileItemContainer container
-                            = new TileItemContainer(game, t);
-                        // TileItemContainer copies every natural item
-                        // including Resource unless importBonuses ==
-                        // false Rumors and roads are not copied
-                        container.copyFrom(importTile.getTileItemContainer(),
-                            importBonuses, true);
-                        t.setTileItemContainer(container);
-                    }
-                    Region r = importTile.getRegion();
+                    && importMap.isValid(x, y)
+                    && (otherTile = importMap.getTile(x, y)) != null
+                    && otherTile.isLand() == landMap.isLand(x, y)) {
+                    t = map.importTile(otherTile, x, y, layer);
+                    Region r = otherTile.getRegion();
                     if (r == null) {
                         fixRegions.add(t);
                     } else {
@@ -939,12 +871,14 @@ public class TerrainGenerator {
                         }
                     }
                 } else {
-                    t = createTile(x, y, landMap, latitude);
+                    final int latitude = map.getLatitude(y);
+                    TileType tt = (landMap.isLand(x, y))
+                        ? getRandomLandTileType(game, latitude)
+                        : getRandomOceanTileType(game, latitude);
+                    t = new Tile(game, tt, x, y);
                 }
-                map.setTile(t, x, y);
-            }
-        }
-        game.setMap(map);
+                return t;
+            });
 
         // Build the regions.
         List<ServerRegion> fixed = ServerRegion.requireFixedRegions(map, lb);
@@ -958,7 +892,7 @@ public class TerrainGenerator {
             map.resetHighSeas(
                 mapOptions.getInteger(MapGeneratorOptions.DISTANCE_TO_HIGH_SEA),
                 mapOptions.getInteger(MapGeneratorOptions.MAXIMUM_DISTANCE_TO_EDGE));
-            if (mapHasLand) {
+            if (landMap.hasLand()) {
                 newRegions.addAll(createMountains(map, lb));
                 newRegions.addAll(createRivers(map, lb));
                 newRegions.addAll(createLakeRegions(map, lb));
@@ -969,18 +903,14 @@ public class TerrainGenerator {
 
         // Connect all new regions to their geographic parent and add to
         // the map.
-        List<ServerRegion> geographic = new ArrayList<>();
-        for (ServerRegion sr : fixed) {
-            if (sr.isGeographic()) geographic.add(sr);
-        }
+        List<ServerRegion> geographic
+            = transform(fixed, ServerRegion::isGeographic);
         for (ServerRegion sr : newRegions) {
-            for (ServerRegion gr : geographic) {
-                if (gr.containsCenter(sr)) {
-                    sr.setParent(gr);
-                    gr.addChild(sr);
-                    gr.setSize(gr.getSize() + sr.getSize());
-                    break;
-                }
+            ServerRegion gr = find(geographic, g -> g.containsCenter(sr));
+            if (gr != null) {
+                sr.setParent(gr);
+                gr.addChild(sr);
+                gr.setSize(gr.getSize() + sr.getSize());
             }
             map.addRegion(sr);
         }
@@ -991,12 +921,10 @@ public class TerrainGenerator {
         // Add the bonuses only after the map is completed.
         // Otherwise we risk creating resources on fields where they
         // do not belong (like sugar in large rivers or tobacco on hills).
-        for (Tile tile : map.getAllTiles()) {
-            perhapsAddBonus(tile, !importBonuses);
-            if (!tile.isLand()) {
-                encodeStyle(tile);
-            }
-        }
+        map.forEachTile(t -> {
+                perhapsAddBonus(t, !importBonuses);
+                if (!t.isLand()) encodeStyle(t);
+            });
 
         // Final cleanups
         map.resetContiguity();
