@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2015   The FreeCol Team
+ *  Copyright (C) 2002-2022   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -19,10 +19,13 @@
 
 package net.sf.freecol.server.model;
 
+import java.util.Comparator;
 import java.util.List;
 import java.util.Random;
+import java.util.function.Predicate;
 import java.util.logging.Logger;
 
+import net.sf.freecol.common.i18n.Messages;
 import net.sf.freecol.common.model.Ability;
 import net.sf.freecol.common.model.Game;
 import net.sf.freecol.common.model.Goods;
@@ -39,19 +42,19 @@ import net.sf.freecol.common.model.Tile;
 import net.sf.freecol.common.model.Turn;
 import net.sf.freecol.common.model.Unit;
 import net.sf.freecol.common.model.UnitType;
+import net.sf.freecol.common.networking.ChangeSet;
+import net.sf.freecol.common.networking.ChangeSet.See;
+import static net.sf.freecol.common.util.CollectionUtils.*;
 import net.sf.freecol.common.util.LogBuilder;
-
 import static net.sf.freecol.common.util.RandomUtils.*;
 
-import net.sf.freecol.server.control.ChangeSet;
-import net.sf.freecol.server.control.ChangeSet.See;
 
 
 /**
  * The server version of an Indian Settlement.
  */
 public class ServerIndianSettlement extends IndianSettlement
-    implements ServerModelObject {
+    implements TurnTaker {
 
     private static final Logger logger = Logger.getLogger(ServerIndianSettlement.class.getName());
 
@@ -65,7 +68,10 @@ public class ServerIndianSettlement extends IndianSettlement
 
 
     /**
-     * Trivial constructor for all ServerModelObjects.
+     * Trivial constructor for Game.newInstance.
+     *
+     * @param game The {@code Game} in which this object belong.
+     * @param id The object identifier.
      */
     public ServerIndianSettlement(Game game, String id) {
         super(game, id);
@@ -74,10 +80,10 @@ public class ServerIndianSettlement extends IndianSettlement
     /**
      * Creates a new ServerIndianSettlement.
      *
-     * @param game The <code>Game</code> in which this object belong.
-     * @param owner The <code>Player</code> owning this settlement.
+     * @param game The {@code Game} in which this object belong.
+     * @param owner The {@code Player} owning this settlement.
      * @param name The name for this settlement.
-     * @param tile The location of the <code>IndianSettlement</code>.
+     * @param tile The location of the {@code IndianSettlement}.
      * @param isCapital True if settlement is tribe's capital
      * @param learnableSkill The skill that can be learned by
      *     Europeans at this settlement.
@@ -101,10 +107,10 @@ public class ServerIndianSettlement extends IndianSettlement
     /**
      * Creates a new ServerIndianSettlement from a template.
      *
-     * @param game The <code>Game</code> in which this object belong.
-     * @param owner The <code>Player</code> owning this settlement.
-     * @param tile The location of the <code>IndianSettlement</code>.
-     * @param template The template <code>IndianSettlement</code> to copy.
+     * @param game The {@code Game} in which this object belong.
+     * @param owner The {@code Player} owning this settlement.
+     * @param tile The location of the {@code IndianSettlement}.
+     * @param template The template {@code IndianSettlement} to copy.
      */
     public ServerIndianSettlement(Game game, Player owner, Tile tile,
                                   IndianSettlement template) {
@@ -128,11 +134,11 @@ public class ServerIndianSettlement extends IndianSettlement
         setLastTribute(template.getLastTribute());
         setGoodsContainer(new GoodsContainer(game, this));
         final Specification spec = getSpecification();
-        for (Goods goods : template.getCompactGoods()) {
+        for (Goods goods : template.getCompactGoodsList()) {
             GoodsType type = spec.getGoodsType(goods.getType().getId());
             addGoods(type, goods.getAmount());
         }
-        wantedGoods = template.getWantedGoods();
+        setWantedGoods(template.getWantedGoods());
     }
 
 
@@ -140,23 +146,22 @@ public class ServerIndianSettlement extends IndianSettlement
      * Starts a new turn for a player.
      *
      * @param random A pseudo-random number source.
-     * @param cs A <code>ChangeSet</code> to update.
+     * @param cs A {@code ChangeSet} to update.
      */
     public void csStartTurn(Random random, ChangeSet cs) {
-        final Specification spec = getSpecification();
         final Unit missionary = getMissionary();
         if (missionary == null) return;
-        final ServerPlayer other = (ServerPlayer)missionary.getOwner();
+        final Player other = missionary.getOwner();
         final Tile tile = getTile();
         final Turn turn = getGame().getTurn();
 
         // Check for braves converted by missionaries
         float convert = getConvertProgress();
-        float cMiss = missionary.applyModifiers(missionary.getType().getSkill(),
-                                                turn, Modifier.CONVERSION_SKILL);
+        float cMiss = missionary.apply(missionary.getType().getSkill(),
+                                       turn, Modifier.CONVERSION_SKILL);
         // The convert rate increases by a percentage of the current alarm.
         int alarm = Math.min(getAlarm(other).getValue(), Tension.TENSION_MAX);
-        float cAlarm = missionary.applyModifiers(alarm, turn,
+        float cAlarm = missionary.apply(alarm, turn,
             Modifier.CONVERSION_ALARM_RATE);
         convert += cMiss + (cAlarm - alarm);
         logger.finest("Conversion at " + getName() + " alarm=" + alarm
@@ -174,10 +179,8 @@ public class ServerIndianSettlement extends IndianSettlement
             // around (as distinct to those with DefendSettlement
             // missions) into the settlement so we can ignore the
             // tile-residents.
-            List<Unit> ul = tile.getUnitList();
-            if (ul.isEmpty()) ul.addAll(getUnitList());
             ServerUnit brave = (ServerUnit)getRandomMember(logger, "Convert",
-                                                           ul, random);
+                getAllUnitsList(), random);
             colony.csAddConvert(brave, cs);
         }
     }
@@ -212,23 +215,301 @@ public class ServerIndianSettlement extends IndianSettlement
         for (int i = 0; i < count; i++) {
             Unit unit = new ServerUnit(game, this, getOwner(), brave,
                                        brave.getDefaultRole());
-            unit.setHomeIndianSettlement(this);
+            unit.changeHomeIndianSettlement(this);
             unit.setLocation(this);
         }
     }
 
+    /**
+     * Convenience function to remove an amount of goods.
+     *
+     * @param type The {@code GoodsType} to remove.
+     * @param amount The amount of goods to remove.
+     */
+    private void consumeGoods(GoodsType type, int amount) {
+        if (getGoodsCount(type) > 0) {
+            amount = Math.min(amount, getGoodsCount(type));
+            removeGoods(type, amount);
+        }
+    }
+
+    /**
+     * Sets alarm towards the given player.
+     *
+     * -til: Might change tile appearance through most hated state
+     *
+     * @param player The {@code Player} to set the alarm level for.
+     * @param newAlarm The new alarm value.
+     */
+    @Override
+    public void setAlarm(Player player, Tension newAlarm) {
+        if (player != null && player != owner) {
+            super.setAlarm(player, newAlarm);
+            updateMostHated();
+        }
+    }
+
+    /**
+     * Removes all alarm towards the given player.  Used the a player leaves
+     * the game.
+     *
+     * -til: Might change tile appearance through most hated state
+     *
+     * @param player The {@code Player} to remove the alarm for.
+     */
+    public void removeAlarm(Player player) {
+        if (player != null) {
+            alarm.remove(player);
+            updateMostHated();
+        }
+    }
+
+    /**
+     * Updates the most hated nation of this settlement.
+     * Needs to be public so it can be set by backwards compatibility code
+     * in FreeColServer.loadGame.
+     *
+     * -til: This might change the tile appearance.
+     *
+     * @return True if the most hated nation changed.
+     */
+    private boolean updateMostHated() {
+        final Player old = this.mostHated;
+        final Predicate<Player> hatedPred = p -> {
+            Tension alarm = getAlarm(p);
+            return alarm != null && alarm.getLevel() != Tension.Level.HAPPY;
+        };
+        final Comparator<Player> mostHatedComp
+            = Comparator.comparingInt(p -> getAlarm(p).getValue());
+        this.mostHated = maximize(getGame().getLiveEuropeanPlayers(),
+                                  hatedPred, mostHatedComp);
+        return this.mostHated != old;
+    }
+
+    /**
+     * Change the alarm level of this settlement by a given amount.
+     *
+     * -til: Might change tile appearance through most hated state
+     *
+     * @param player The {@code Player} the alarm level changes wrt.
+     * @param amount The amount to change the alarm by.
+     * @return True if the {@code Tension.Level} of the
+     *     settlement alarm changes as a result of this change.
+     */
+    private boolean changeAlarm(Player player, int amount) {
+        Tension alarm = getAlarm(player);
+        if (alarm == null) {
+            initializeAlarm(player);
+            alarm = getAlarm(player);
+        }
+        Tension.Level oldLevel = alarm.getLevel();
+        alarm.modify(amount);
+        boolean change = updateMostHated();
+        return change || oldLevel != alarm.getLevel();
+    }
+
+    /**
+     * Modifies the alarm level towards the given player due to an event
+     * at this settlement, and propagate the alarm upwards through the
+     * tribe.
+     *
+     * -til: Might change tile appearance through most hated state
+     *
+     * @param player The {@code Player} to modify alarm for.
+     * @param add The amount to add to the current alarm level.
+     * @param propagate If true, propagate the alarm change upward to the
+     *     owning player.
+     * @param cs A {@code ChangeSet} to update.
+     * @return True if the alarm changed.
+     */
+    private boolean csChangeAlarm(Player player, int add, boolean propagate,
+                                  ChangeSet cs) {
+        boolean change = changeAlarm(player, add);
+        if (propagate) {
+            // Propagate alarm upwards.  Capital has a greater impact.
+            ((ServerPlayer)getOwner()).csModifyTension(player,
+                ((isCapital()) ? add : add/2), this, cs);
+        }
+        logger.finest("Alarm at " + getName()
+            + " toward " + player.getName()
+            + " modified by " + add
+            + " now = " + getAlarm(player).getValue());
+        return change;
+    }
+
+    /**
+     * Modifies the alarm level towards the given player due to an event
+     * at this settlement, and propagate the alarm upwards through the
+     * tribe.
+     *
+     * +til: Handles tile visibility changes.
+     *
+     * @param player The {@code Player} to modify alarm for.
+     * @param add The amount to add to the current alarm level.
+     * @param propagate If true, propagate the alarm change upward to the
+     *     owning player.
+     * @param cs A {@code ChangeSet} to update.
+     * @return True if the alarm changed and the tile added.
+     */
+    public boolean csModifyAlarm(Player player, int add, boolean propagate,
+                                 ChangeSet cs) {
+        Tile copied = getTile().getTileToCache();
+        boolean change = csChangeAlarm(player, add, propagate, cs);//-til
+        if (change) {
+            getTile().cacheUnseen(copied);//+til
+            cs.add(See.perhaps(), this);
+        }
+        return change;
+    }
+
+    /**
+     * Changes the missionary for this settlement and updates other players.
+     *
+     * +vis: Handles the visibility implications.
+     * +til: Handles the tile appearance change.
+     *
+     * @param missionary The new missionary for this settlement.
+     * @param cs A {@code ChangeSet} to update.
+     */
+    public void csChangeMissionary(Unit missionary, ChangeSet cs) {
+        final Unit old = getMissionary();
+        if (missionary == old) return;
+
+        final Tile tile = getTile();
+        final Player newOwner = (missionary == null) ? null
+            : missionary.getOwner();
+        tile.cacheUnseen(newOwner);//+til
+
+        if (old != null) {
+            final Player oldOwner = old.getOwner(); 
+            setMissionary(null);//-vis(oldOwner),-til
+            tile.updateIndianSettlement(oldOwner);
+            ((ServerUnit)old).csRemove(See.only(oldOwner),
+                null, cs);//-vis(oldOwner)
+            cs.add(See.only(oldOwner), tile);
+            oldOwner.invalidateCanSeeTiles();//+vis(oldOwner)
+        }
+
+        if (missionary != null) {
+            setMissionary(missionary);//-vis(newOwner)
+            // Take the missionary off the map, and give it a fake
+            // location at the settlement, bypassing the normal
+            // validity checks.
+            missionary.setLocation(null);//-vis(newOwner)
+            missionary.setLocationNoUpdate(this);//-vis(newOwner),-til
+            missionary.setMovesLeft(0);
+            cs.add(See.only(newOwner), missionary);
+            setConvertProgress(0);
+            csChangeAlarm(newOwner, ALARM_NEW_MISSIONARY, true, cs);//-til
+            tile.updateIndianSettlement(newOwner);
+
+            cs.add(See.only(newOwner),
+                ((ServerPlayer)newOwner).collectNewTiles(getMissionaryVisibleTiles()));
+            cs.add(See.perhaps().always(newOwner), tile);
+            newOwner.invalidateCanSeeTiles();//+vis(newOwner)
+        }
+    }
+
+    /**
+     * Kills the missionary at this settlement.
+     *
+     * @param destroy If true, the settlement is destroyed, if false the
+     *     missionary is denounced, if null do not generate a message.
+     * @param cs A {@code ChangeSet} to update.
+     */
+    public void csKillMissionary(Boolean destroy, ChangeSet cs) {
+        Unit missionary = getMissionary();
+        if (missionary == null) return;
+        csChangeMissionary(null, cs);
+        
+        // Inform the enemy of loss of mission
+        Player missionaryOwner = missionary.getOwner();
+        if (destroy != null) {
+            if (destroy) {
+                cs.addMessage(missionaryOwner,
+                    new ModelMessage(ModelMessage.MessageType.UNIT_LOST,
+                                     "model.indianSettlement.mission.destroyed",
+                                     this)
+                        .addStringTemplate("%settlement%",
+                            getLocationLabelFor(missionaryOwner)));
+            } else {
+                cs.addMessage(missionaryOwner,
+                    new ModelMessage(ModelMessage.MessageType.FOREIGN_DIPLOMACY,
+                                     "model.indianSettlement.mission.denounced",
+                                     this)
+                        .addStringTemplate("%settlement%",
+                            getLocationLabelFor(missionaryOwner)));
+            }
+        }
+    }
+
+    /**
+     * Equip a unit for a specific role.
+     *
+     * @param unit The {@code Unit} to equip.
+     * @param role The {@code Role} to equip for.
+     * @param roleCount The role count.
+     * @param random A pseudo-random number source.
+     * @param cs A {@code ChangeSet} to update.
+     * @return True if the equipping succeeds.
+     */
+    public boolean csEquipForRole(Unit unit, Role role, int roleCount,
+                                  Random random, ChangeSet cs) {
+        boolean ret = equipForRole(unit, role, roleCount);
+
+        if (ret) {
+            cs.add(See.only(getOwner()), getTile());
+        }
+        return ret;
+    }
+
+    /**
+     * Check if tension has increased with respect to an enemy.
+     *
+     * @param enemy The enemy {@code Player}.
+     * @param oldLevel The previous tension {@code Level}.
+     * @param cs A {@code ChangeSet} to update.
+     */
+    public void csCheckTension(Player enemy, Tension.Level oldLevel,
+                               ChangeSet cs) {
+        Tension newTension = getAlarm(enemy);
+        Tension.Level newLevel = (newTension == null) ? null
+            : newTension.getLevel();
+        if (oldLevel == null || oldLevel == newLevel
+            || !hasContacted(enemy) || !enemy.hasExplored(getTile())) return;
+
+        // Tension has changed, update the enemy
+        cs.add(See.only(null).perhapsOnly(enemy), this);
+
+        // No messages about improving tension
+        if (newLevel == null
+            || oldLevel.getLimit() > newLevel.getLimit()) return;
+
+        // Send a message to the enemy that tension has increased.
+        String key = "model.player.alarmIncrease." + getAlarm(enemy).getKey();
+        if (!Messages.containsKey(key)) return;
+        cs.addMessage(enemy,
+            new ModelMessage(ModelMessage.MessageType.FOREIGN_DIPLOMACY,
+                             key, this)
+                .addStringTemplate("%nation%", getOwner().getNationLabel())
+                .addStringTemplate("%enemy%", enemy.getNationLabel())
+                .addName("%settlement%", getName()));
+    }
+        
+    // Implement TurnTaker
+
     /**
      * New turn for this native settlement.
      *
-     * @param random A <code>Random</code> number source.
-     * @param lb A <code>LogBuilder</code> to log to.
-     * @param cs A <code>ChangeSet</code> to update.
+     * @param random A {@code Random} number source.
+     * @param lb A {@code LogBuilder} to log to.
+     * @param cs A {@code ChangeSet} to update.
      */
     @Override
     public void csNewTurn(Random random, LogBuilder lb, ChangeSet cs) {
+        final Specification spec = getSpecification();
+        final Player owner = getOwner();
         lb.add(this);
-        ServerPlayer owner = (ServerPlayer) getOwner();
-        Specification spec = getSpecification();
 
         // Produce goods.
         List<GoodsType> goodsList = spec.getGoodsTypeList();
@@ -245,15 +526,15 @@ public class ServerIndianSettlement extends IndianSettlement
         int storedFood = getGoodsCount(spec.getPrimaryFoodType());
         if (storedFood <= 0 && getUnitCount() > 0) {
             Unit victim = getRandomMember(logger, "Choose starver",
-                                          getUnitList(), random);
-            cs.addRemove(See.only(owner), this, victim);//-vis(owner)
-            victim.dispose();
+                                          getUnits(), random);
+            ((ServerUnit)victim).csRemove(See.only(owner),
+                this, cs);//-vis(owner)
             lb.add(" FAMINE");
         }
         if (getUnitCount() <= 0) {
             if (tile.isEmpty()) {
                 lb.add(" COLLAPSED, ");
-                owner.csDisposeSettlement(this, cs);//+vis(owner)
+                ((ServerPlayer)owner).csDisposeSettlement(this, cs);//+vis(owner)
                 return;
             }
             tile.getFirstUnit().setLocation(this);//-vis,til: safe in settlement
@@ -280,7 +561,7 @@ public class ServerIndianSettlement extends IndianSettlement
                 consumeGoods(rumType, FOOD_PER_COLONIST/4);
                 // New units quickly go out of their city and start annoying.
                 addOwnedUnit(unit);
-                unit.setHomeIndianSettlement(this);
+                unit.changeHomeIndianSettlement(this);
                 lb.add(" new ", unit);
             }
             // Consume the food anyway
@@ -293,8 +574,8 @@ public class ServerIndianSettlement extends IndianSettlement
         // FIXME: remove this
         GoodsType grainType = spec.getGoodsType("model.goods.grain");
         int foodProdAvail = getTotalProductionOf(grainType) - getFoodConsumption();
-        if (getGoodsCount(horsesType) >= horsesType.getBreedingNumber()
-            && foodProdAvail > 0) {
+        if (foodProdAvail > 0
+            && getGoodsCount(horsesType) >= horsesType.getBreedingNumber()) {
             int nHorses = Math.min(MAX_HORSES_PER_TURN, foodProdAvail);
             addGoods(horsesType, nHorses);
             lb.add(" bred ", nHorses, " horses");
@@ -305,264 +586,4 @@ public class ServerIndianSettlement extends IndianSettlement
         cs.add(See.only(owner), this);
         lb.add(", ");
     }
-
-    /**
-     * Convenience function to remove an amount of goods.
-     *
-     * @param type The <code>GoodsType</code> to remove.
-     * @param amount The amount of goods to remove.
-     */
-    private void consumeGoods(GoodsType type, int amount) {
-        if (getGoodsCount(type) > 0) {
-            amount = Math.min(amount, getGoodsCount(type));
-            removeGoods(type, amount);
-        }
-    }
-
-    /**
-     * Sets alarm towards the given player.
-     *
-     * -til: Might change tile appearance through most hated state
-     *
-     * @param player The <code>Player</code> to set the alarm level for.
-     * @param newAlarm The new alarm value.
-     */
-    @Override
-    public void setAlarm(Player player, Tension newAlarm) {
-        if (player != null && player != owner) {
-            super.setAlarm(player, newAlarm);
-            updateMostHated();
-        }
-    }
-
-    /**
-     * Removes all alarm towards the given player.  Used the a player leaves
-     * the game.
-     *
-     * -til: Might change tile appearance through most hated state
-     *
-     * @param player The <code>Player</code> to remove the alarm for.
-     */
-    public void removeAlarm(Player player) {
-        if (player != null) {
-            alarm.remove(player);
-            updateMostHated();
-        }
-    }
-
-    /**
-     * Updates the most hated nation of this settlement.
-     * Needs to be public so it can be set by backwards compatibility code
-     * in FreeColServer.loadGame.
-     *
-     * -til: This might change the tile appearance.
-     *
-     * @return True if the most hated nation changed.
-     */
-    public boolean updateMostHated() {
-        Player old = mostHated;
-        mostHated = null;
-        int bestValue = Integer.MIN_VALUE;
-        for (Player p : getGame().getLiveEuropeanPlayers(null)) {
-            Tension alarm = getAlarm(p);
-            if (alarm == null
-                || alarm.getLevel() == Tension.Level.HAPPY) continue;
-            int value = alarm.getValue();
-            if (bestValue < value) {
-                bestValue = value;
-                mostHated = p;
-            }
-        }
-        return mostHated != old;
-    }
-
-    /**
-     * Change the alarm level of this settlement by a given amount.
-     *
-     * -til: Might change tile appearance through most hated state
-     *
-     * @param player The <code>Player</code> the alarm level changes wrt.
-     * @param amount The amount to change the alarm by.
-     * @return True if the <code>Tension.Level</code> of the
-     *     settlement alarm changes as a result of this change.
-     */
-    private boolean changeAlarm(Player player, int amount) {
-        Tension alarm = getAlarm(player);
-        if (alarm == null) {
-            initializeAlarm(player);
-            alarm = getAlarm(player);
-        }
-        Tension.Level oldLevel = alarm.getLevel();
-        alarm.modify(amount);
-        boolean change = updateMostHated();
-        return change || oldLevel != alarm.getLevel();
-    }
-
-    /**
-     * Modifies the alarm level towards the given player due to an event
-     * at this settlement, and propagate the alarm upwards through the
-     * tribe.
-     *
-     * -til: Might change tile appearance through most hated state
-     *
-     * @param player The <code>Player</code> to modify alarm for.
-     * @param add The amount to add to the current alarm level.
-     * @param propagate If true, propagate the alarm change upward to the
-     *     owning player.
-     * @param cs A <code>ChangeSet</code> to update.
-     * @return True if the alarm changed.
-     */
-    private boolean csChangeAlarm(Player player, int add, boolean propagate,
-                                  ChangeSet cs) {
-        boolean change = changeAlarm(player, add);
-        if (propagate) {
-            // Propagate alarm upwards.  Capital has a greater impact.
-            ((ServerPlayer)getOwner()).csModifyTension(player,
-                ((isCapital()) ? add : add/2), this, cs);
-        }
-        logger.finest("Alarm at " + getName()
-            + " toward " + player.getName()
-            + " modified by " + add
-            + " now = " + getAlarm(player).getValue());
-        return change;
-    }
-
-    /**
-     * Modifies the alarm level towards the given player due to an event
-     * at this settlement, and propagate the alarm upwards through the
-     * tribe.
-     *
-     * +til: Handles tile visibility changes.
-     *
-     * @param player The <code>Player</code> to modify alarm for.
-     * @param add The amount to add to the current alarm level.
-     * @param propagate If true, propagate the alarm change upward to the
-     *     owning player.
-     * @param cs A <code>ChangeSet</code> to update.
-     * @return True if the alarm changed and the tile added.
-     */
-    public boolean csModifyAlarm(Player player, int add, boolean propagate,
-                                 ChangeSet cs) {
-        Tile copied = getTile().getTileToCache();
-        boolean change = csChangeAlarm(player, add, propagate, cs);//-til
-        if (change) {
-            getTile().cacheUnseen(copied);//+til
-            cs.add(See.perhaps(), this);
-        }
-        return change;
-    }
-
-    /**
-     * Changes the missionary for this settlement and updates other players.
-     *
-     * +vis: Handles the visibility implications.
-     * +til: Handles the tile appearance change.
-     *
-     * @param missionary The new missionary for this settlement.
-     * @param cs A <code>ChangeSet</code> to update.
-     */
-    public void csChangeMissionary(Unit missionary, ChangeSet cs) {
-        final Unit old = getMissionary();
-        if (missionary == old) return;
-
-        final Tile tile = getTile();
-        final ServerPlayer newOwner = (missionary == null) ? null
-            : (ServerPlayer)missionary.getOwner();
-        tile.cacheUnseen(newOwner);//+til
-
-        if (old != null) {
-            final ServerPlayer oldOwner = (ServerPlayer)old.getOwner(); 
-            setMissionary(null);//-vis(oldOwner),-til
-            tile.updateIndianSettlement(oldOwner);
-            cs.addRemove(See.only(oldOwner), null, old);//-vis(oldOwner)
-            old.dispose();
-            cs.add(See.only(oldOwner), tile);
-            oldOwner.invalidateCanSeeTiles();//+vis(oldOwner)
-        }
-
-        if (missionary != null) {
-            setMissionary(missionary);//-vis(newOwner)
-            // Take the missionary off the map, and give it a fake
-            // location at the settlement, bypassing the normal
-            // validity checks.
-            missionary.setLocation(null);//-vis(newOwner)
-            missionary.setLocationNoUpdate(this);//-vis(newOwner),-til
-            missionary.setMovesLeft(0);
-            cs.add(See.only(newOwner), missionary);
-            setConvertProgress(0);
-            csChangeAlarm(newOwner, ALARM_NEW_MISSIONARY, true, cs);//-til
-            tile.updateIndianSettlement(newOwner);
-            
-            int radius = getMissionaryLineOfSight();
-            for (Tile t : tile.getSurroundingTiles(1, radius)) {
-                if (newOwner.exploreTile(t) || !newOwner.canSee(t)) {
-                    cs.add(See.only(newOwner), t);
-                }
-            }
-            cs.add(See.perhaps().always(newOwner), tile);
-            newOwner.invalidateCanSeeTiles();//+vis(newOwner)
-        }
-    }
-
-    /**
-     * Kills the missionary at this settlement.
-     *
-     * @param destroy If true, the settlement is destroyed, if false the
-     *     missionary is denounced, if null do not generate a message.
-     * @param cs A <code>ChangeSet</code> to update.
-     */
-    public void csKillMissionary(Boolean destroy, ChangeSet cs) {
-        Unit missionary = getMissionary();
-        if (missionary == null) return;
-        csChangeMissionary(null, cs);
-        
-        // Inform the enemy of loss of mission
-        ServerPlayer missionaryOwner = (ServerPlayer)missionary.getOwner();
-        if (destroy != null) {
-            if (destroy) {
-                cs.addMessage(See.only(missionaryOwner),
-                    new ModelMessage(ModelMessage.MessageType.UNIT_LOST,
-                        "model.indianSettlement.mission.destroyed", this)
-                        .addStringTemplate("%settlement%",
-                            getLocationLabelFor(missionaryOwner)));
-            } else {
-                cs.addMessage(See.only(missionaryOwner),
-                    new ModelMessage(ModelMessage.MessageType.FOREIGN_DIPLOMACY,
-                        "model.indianSettlement.mission.denounced", this)
-                        .addStringTemplate("%settlement%",
-                            getLocationLabelFor(missionaryOwner)));
-            }
-        }
-    }
-
-    /**
-     * Equip a unit for a specific role.
-     *
-     * @param unit The <code>Unit</code> to equip.
-     * @param role The <code>Role</code> to equip for.
-     * @param roleCount The role count.
-     * @param random A pseudo-random number source.
-     * @param cs A <code>ChangeSet</code> to update.
-     * @return True if the equipping succeeds.
-     */
-    public boolean csEquipForRole(Unit unit, Role role, int roleCount,
-                                  Random random, ChangeSet cs) {
-        boolean ret = equipForRole(unit, role, roleCount);
-
-        if (ret) {
-            cs.add(See.only((ServerPlayer)getOwner()), getTile());
-        }
-        return ret;
-    }
-
-
-    /**
-     * Gets the tag name of the root element representing this object.
-     *
-     * @return "serverIndianSettlement"
-     */
-    @Override
-    public String getServerXMLElementTagName() {
-        return "serverIndianSettlement";
-    }
 }
