@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2015  The FreeCol Team
+ *  Copyright (C) 2002-2022  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -32,7 +32,7 @@ public class AllTests {
         suite.addTest(net.sf.freecol.common.option.AllTests.suite());
         suite.addTest(net.sf.freecol.common.model.AllTests.suite());
         suite.addTest(net.sf.freecol.common.utils.AllTests.suite());
-        suite.addTest(net.sf.freecol.common.sound.AllTests.suite());
+        // suite.addTest(net.sf.freecol.common.sound.AllTests.suite());
         //$JUnit-END$
         return suite;
     }
