@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2015  The FreeCol Team
+ *  Copyright (C) 2002-2022  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -24,12 +24,12 @@ import java.util.List;
 import java.util.Random;
 import java.util.Set;
 
-import net.sf.freecol.common.model.CombatModel;
 import net.sf.freecol.common.model.CombatModel.CombatResult;
 import net.sf.freecol.common.model.Unit.MoveType;
-import net.sf.freecol.common.option.IntegerOption;
+import net.sf.freecol.common.networking.ChangeSet;
+import net.sf.freecol.common.option.GameOptions;
+import static net.sf.freecol.common.util.CollectionUtils.*;
 import net.sf.freecol.server.ServerTestHelper;
-import net.sf.freecol.server.control.ChangeSet;
 import net.sf.freecol.server.control.InGameController;
 import net.sf.freecol.server.model.ServerPlayer;
 import net.sf.freecol.server.model.ServerUnit;
@@ -91,11 +91,12 @@ public class CombatTest extends FreeColTestCase {
 
     public void testColonistAttackedByVeteran() throws Exception {
         Game game = getStandardGame();
+        Map map = getTestMap(plains);
+        game.changeMap(map);
+
         CombatModel combatModel = game.getCombatModel();
         Player dutch = game.getPlayerByNationId("model.nation.dutch");
         Player french = game.getPlayerByNationId("model.nation.french");
-        Map map = getTestMap(plains);
-        game.setMap(map);
         FreeColTestCase.spec();
         Tile tile1 = map.getTile(5, 8);
         tile1.setType(hills);
@@ -115,13 +116,13 @@ public class CombatTest extends FreeColTestCase {
         assertFalse(colonist.hasAbility(Ability.AMBUSH_PENALTY));
 
         final Modifier bigMovementPenalty
-            = spec().getModifiers(Modifier.BIG_MOVEMENT_PENALTY).get(0);
+            = first(spec().getModifiers(Modifier.BIG_MOVEMENT_PENALTY));
         final Modifier attackModifier
-            = spec().getModifiers(Modifier.ATTACK_BONUS).get(0);
-        final Set<Modifier> veteranModifierSet
-            = veteranType.getModifiers(Modifier.OFFENCE);
-        assertEquals(1, veteranModifierSet.size());
-        final Modifier veteranModifier = veteranModifierSet.iterator().next();
+            = first(spec().getModifiers(Modifier.ATTACK_BONUS));
+        final List<Modifier> veteranModifiers
+            = toList(veteranType.getModifiers(Modifier.OFFENCE));
+        assertEquals(1, veteranModifiers.size());
+        final Modifier veteranModifier = first(veteranModifiers);
 
         Set<Modifier> offenceModifiers
             = combatModel.getOffensiveModifiers(soldier, colonist);
@@ -137,10 +138,10 @@ public class CombatTest extends FreeColTestCase {
         assertEquals(31, n);
 
         final Modifier fortifiedModifier
-            = spec().getModifiers(Modifier.FORTIFIED).get(0);
-        final Set<Modifier> hillsModifierSet = hills.getDefenceModifiers();
-        assertEquals(1, hillsModifierSet.size());
-        Modifier hillsModifier = hillsModifierSet.iterator().next();
+            = first(spec().getModifiers(Modifier.FORTIFIED));
+        final List<Modifier> hillsModifiers = toList(hills.getDefenceModifiers());
+        assertEquals(1, hillsModifiers.size());
+        Modifier hillsModifier = first(hillsModifiers);
 
         Set<Modifier> defenceModifiers
             = combatModel.getDefensiveModifiers(soldier, colonist);
@@ -151,16 +152,17 @@ public class CombatTest extends FreeColTestCase {
         defenceModifiers.remove(hillsModifier);
         // this was also added by the combat model
         assertEquals(Specification.BASE_DEFENCE_SOURCE,
-            defenceModifiers.iterator().next().getSource());
+            first(defenceModifiers).getSource());
     }
 
     public void testGalleonAttackedByPrivateer() throws Exception {
         Game game = getStandardGame();
+        Map map = getTestMap(ocean);
+        game.changeMap(map);
+
         CombatModel combatModel = game.getCombatModel();
         Player dutch = game.getPlayerByNationId("model.nation.dutch");
         Player french = game.getPlayerByNationId("model.nation.french");
-        Map map = getTestMap(ocean);
-        game.setMap(map);
         FreeColTestCase.spec();
         Tile tile1 = map.getTile(5, 8);
         tile1.setExplored(dutch, true);
@@ -186,7 +188,8 @@ public class CombatTest extends FreeColTestCase {
 
         Set<Modifier> defenceModifiers = combatModel.getDefensiveModifiers(privateer, galleon);
         assertEquals(1, defenceModifiers.size());
-        assertEquals(Specification.BASE_DEFENCE_SOURCE, defenceModifiers.iterator().next().getSource());
+        assertEquals(Specification.BASE_DEFENCE_SOURCE,
+                     first(defenceModifiers).getSource());
 
         /**
          * Fortification should have no effect.
@@ -194,7 +197,8 @@ public class CombatTest extends FreeColTestCase {
         galleon.setStateUnchecked(Unit.UnitState.FORTIFIED);
         defenceModifiers = combatModel.getDefensiveModifiers(privateer, galleon);
         assertEquals(1, defenceModifiers.size());
-        assertEquals(Specification.BASE_DEFENCE_SOURCE, defenceModifiers.iterator().next().getSource());
+        assertEquals(Specification.BASE_DEFENCE_SOURCE,
+                     first(defenceModifiers).getSource());
 
         /**
          * Penalties due to cargo.
@@ -229,11 +233,11 @@ public class CombatTest extends FreeColTestCase {
 
         // Francis Drake
         FoundingFather drake = spec().getFoundingFather("model.foundingFather.francisDrake");
-        Set<Modifier> drakeModifiers = drake.getModifiers(Modifier.OFFENCE, privateerType);
-        assertEquals(1, drakeModifiers.size());
+        Modifier drakeModifier = first(drake.getModifiers(Modifier.OFFENCE, privateerType));
+        assertNotNull(drakeModifier);
         french.addFather(drake);
-        drakeModifiers = french.getModifiers(Modifier.OFFENCE, privateerType);
-        assertEquals(1, drakeModifiers.size());
+        assertEquals(drakeModifier, first(french.getModifiers(Modifier.OFFENCE, privateerType)));
+
         offenceModifiers = combatModel.getOffensiveModifiers(privateer, galleon);
         assertEquals(4, offenceModifiers.size());
         n = 0;
@@ -256,7 +260,7 @@ public class CombatTest extends FreeColTestCase {
     public void testDefendColonyWithUnarmedColonist() {
         Game game = getGame();
         Map map = getTestMap(true);
-        game.setMap(map);
+        game.changeMap(map);
 
         Colony colony = getStandardColony();
         Player dutch = game.getPlayerByNationId("model.nation.dutch");
@@ -265,7 +269,7 @@ public class CombatTest extends FreeColTestCase {
         Tile tile2 = map.getTile(4, 8);
         tile2.setExplored(dutch, true);
 
-        Unit colonist = colony.getUnitIterator().next();
+        Unit colonist = first(colony.getUnitList());
         Unit attacker = new ServerUnit(getGame(), tile2, inca, braveType,
                                        armedBraveRole);
 
@@ -282,7 +286,7 @@ public class CombatTest extends FreeColTestCase {
     public void testDefendColonyWithRevere() {
         final Game game = getGame();
         final Map map = getTestMap(true);
-        game.setMap(map);
+        game.changeMap(map);
 
         final SimpleCombatModel combatModel = new SimpleCombatModel();
         Player dutch = game.getPlayerByNationId("model.nation.dutch");
@@ -290,7 +294,7 @@ public class CombatTest extends FreeColTestCase {
         Colony colony = getStandardColony();
         Tile tile2 = map.getTile(4, 8);
         tile2.setExplored(dutch, true);
-        Unit colonist = colony.getUnitIterator().next();
+        Unit colonist = first(colony.getUnitList());
         Unit attacker = new ServerUnit(getGame(), tile2, inca, braveType,
                                        armedBraveRole);
 
@@ -299,7 +303,7 @@ public class CombatTest extends FreeColTestCase {
 
         // Set up for auto-equip
         dutch.addFather(spec().getFoundingFather("model.foundingFather.paulRevere"));
-        for (AbstractGoods ag : soldierRole.getRequiredGoods()) {
+        for (AbstractGoods ag : soldierRole.getRequiredGoodsList()) {
             colony.addGoods(ag);
         }
 
@@ -308,15 +312,14 @@ public class CombatTest extends FreeColTestCase {
 
         Set<Modifier> defenceModifiers = combatModel
             .getDefensiveModifiers(attacker, colonist);
-        for (Modifier modifier : soldierRole.getModifiers(Modifier.DEFENCE)) {
-            assertTrue(defenceModifiers.contains(modifier));
-        }
+        forEach(soldierRole.getModifiers(Modifier.DEFENCE),
+                m -> assertTrue(defenceModifiers.contains(m)));
     }
 
     public void testDefendSettlement() {
         Game game = getStandardGame();
         Map map = getTestMap();
-        game.setMap(map);
+        game.changeMap(map);
 
         SimpleCombatModel combatModel = new SimpleCombatModel();
         Player dutch = game.getPlayerByNationId("model.nation.dutch");
@@ -330,30 +333,30 @@ public class CombatTest extends FreeColTestCase {
 
         FreeColTestCase.IndianSettlementBuilder builder
             = new FreeColTestCase.IndianSettlementBuilder(game);
-        IndianSettlement settlement = builder.player(inca)
+        IndianSettlement is = builder.player(inca)
             .settlementTile(tile1).skillToTeach(null).capital(true).build();
 
-        Unit defender = new ServerUnit(game, settlement, inca, braveType,
+        Unit defender = new ServerUnit(game, is, inca, braveType,
                                        nativeDragoonRole);
         Unit attacker = new ServerUnit(game, tile2, dutch, colonistType,
                                        dragoonRole);
-        for (AbstractGoods ag : nativeDragoonRole.getRequiredGoods()) {
-            settlement.addGoods(ag);
+        for (AbstractGoods ag : nativeDragoonRole.getRequiredGoodsList()) {
+            is.addGoods(ag);
         }
 
         Set<Modifier> defenceModifiers = combatModel
             .getDefensiveModifiers(attacker, defender);
-        for (Modifier modifier : nativeDragoonRole.getModifiers(Modifier.DEFENCE)) {
-            assertTrue(defenceModifiers.contains(modifier));
-        }
+        forEach(nativeDragoonRole.getModifiers(Modifier.DEFENCE),
+                m -> assertTrue(defenceModifiers.contains(m)));
     }
 
     public void testAttackIgnoresMovementPoints() throws Exception {
         Game game = getStandardGame();
+        Map map = getTestMap(plains, true);
+        game.changeMap(map);
+
         Player dutch = game.getPlayerByNationId("model.nation.dutch");
         Player french = game.getPlayerByNationId("model.nation.french");
-        Map map = getTestMap(plains, true);
-        game.setMap(map);
         Tile tile1 = map.getTile(5, 8);
         Tile tile2 = map.getTile(4, 8);
         tile1.setType(hills);
@@ -381,11 +384,12 @@ public class CombatTest extends FreeColTestCase {
 
     public void testSpanishAgainstNatives() throws Exception {
         final Game game = getStandardGame();
+        Map map = getTestMap(plains, true);
+        game.changeMap(map);
+
         final Player spanish = game.getPlayerByNationId("model.nation.spanish");
         final Player dutch = game.getPlayerByNationId("model.nation.dutch");
         Player tupi = game.getPlayerByNationId("model.nation.tupi");
-        Map map = getTestMap(plains, true);
-        game.setMap(map);
 
         SimpleCombatModel combatModel = new SimpleCombatModel();
 
@@ -434,13 +438,12 @@ public class CombatTest extends FreeColTestCase {
 
     public void testAttackShipWithLandUnit() {
         Game game = getStandardGame();
+        Map map = getTestMap(plains, true);
+        game.changeMap(map);
+
         Player spanish = game.getPlayerByNationId("model.nation.spanish");
         Player tupi = game.getPlayerByNationId("model.nation.tupi");
-        Map map = getTestMap(plains, true);
-        game.setMap(map);
-
         SimpleCombatModel combatModel = new SimpleCombatModel();
-
         Tile tile1 = map.getTile(5, 8);
         Tile tile2 = map.getTile(4, 8);
 
@@ -470,7 +473,7 @@ public class CombatTest extends FreeColTestCase {
         Game game = ServerTestHelper.startServerGame(getTestMap(plains, true));
         InGameController igc = ServerTestHelper.getInGameController();
 
-        ServerPlayer french = (ServerPlayer)game.getPlayerByNationId("model.nation.french");
+        ServerPlayer french = getServerPlayer(game, "model.nation.french");
         french.addAbility(new Ability(Ability.INDEPENDENCE_DECLARED));
         ServerPlayer refPlayer = igc.createREFPlayer(french);
 
@@ -546,8 +549,8 @@ public class CombatTest extends FreeColTestCase {
         CombatModel combatModel = game.getCombatModel();
         InGameController igc = ServerTestHelper.getInGameController();
 
-        ServerPlayer dutch = (ServerPlayer)game.getPlayerByNationId("model.nation.dutch");
-        ServerPlayer inca = (ServerPlayer)game.getPlayerByNationId("model.nation.inca");
+        ServerPlayer dutch = getServerPlayer(game, "model.nation.dutch");
+        ServerPlayer inca = getServerPlayer(game, "model.nation.inca");
         dutch.setStance(inca, Stance.WAR);
         inca.setStance(dutch, Stance.WAR);
 
@@ -559,17 +562,15 @@ public class CombatTest extends FreeColTestCase {
                                          jesuitMissionaryType, missionaryRole);
         FreeColTestCase.IndianSettlementBuilder builder
             = new FreeColTestCase.IndianSettlementBuilder(game);
-        IndianSettlement settlement = builder.player(inca)
+        IndianSettlement is = builder.player(inca)
             .settlementTile(tile1).skillToTeach(null).capital(true)
             .initialBravesInCamp(8).missionary(missionary).build();
 
-        IntegerOption io = spec().getIntegerOption("model.option.nativeConvertProbability");
-        assertNotNull(io);
-        io.setValue(100);
+        spec().setInteger(GameOptions.NATIVE_CONVERT_PROBABILITY, 100);
 
         Unit soldier = new ServerUnit(game, tile2, dutch, veteranType,
                                       soldierRole);
-        Unit defender = settlement.getDefendingUnit(soldier);
+        Unit defender = is.getDefendingUnit(soldier);
         assertNotNull(defender);
         assertTrue(defender.getOwner().isIndian());
         
@@ -587,7 +588,7 @@ public class CombatTest extends FreeColTestCase {
         dutch.csCombat(soldier, defender, crs, new Random(),
                        new ChangeSet());
         assertEquals("Two units on tile", 2, tile2.getUnitList().size());
-        assertEquals("Convert on tile", tile2.getUnitList().get(1).getType(),
-                     indianConvertType);
+        assertEquals("Convert on tile", indianConvertType,
+                     tile2.getUnitList().get(1).getType());
     }
 }
