@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2015  The FreeCol Team
+ *  Copyright (C) 2002-2022  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -19,17 +19,22 @@
 
 package net.sf.freecol.common.model;
 
+import static net.sf.freecol.common.util.CollectionUtils.first;
+import static net.sf.freecol.common.util.CollectionUtils.forEachMapEntry;
+
 import java.io.ByteArrayInputStream;
 import java.io.File;
+import java.io.IOException;
 import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
-import java.util.Map.Entry;
+
+import javax.xml.stream.XMLStreamException;
 
 import net.sf.freecol.common.io.FreeColModFile;
+import net.sf.freecol.common.io.FreeColRules;
 import net.sf.freecol.common.io.FreeColTcFile;
-import net.sf.freecol.common.model.Modifier;
 import net.sf.freecol.common.model.Modifier.ModifierType;
 import net.sf.freecol.util.test.FreeColTestCase;
 
@@ -88,7 +93,7 @@ public final class SpecificationTest extends FreeColTestCase {
     public void testLoad() {
         Specification spec = null;
         try {
-            spec = new FreeColTcFile("freecol").getSpecification();
+            spec = FreeColRules.getFreeColRulesFile("classic").getSpecification();
         } catch (Exception e) {
             e.printStackTrace();
             fail();
@@ -167,13 +172,12 @@ public final class SpecificationTest extends FreeColTestCase {
         */
 
         // Additive Modifier
-        Modifier modifier = depotType
-            .getModifiers(Modifier.WAREHOUSE_STORAGE).iterator().next();
+        Modifier modifier = first(depotType.getModifiers(Modifier.WAREHOUSE_STORAGE));
         assertEquals(ModifierType.ADDITIVE, modifier.getType());
         assertEquals(100f, modifier.getValue());
 
         // Multiplicative Modifier
-        modifier = blackSmithType.getModifiers("model.goods.tools").iterator().next();
+        modifier = first(blackSmithType.getModifiers("model.goods.tools"));
         assertEquals(ModifierType.MULTIPLICATIVE, modifier.getType());
         assertEquals(2f, modifier.getValue());
     }
@@ -273,77 +277,76 @@ public final class SpecificationTest extends FreeColTestCase {
         roleAbilities.put("model.role.nativeDragoon", expectAbilities);
 
         // Verify
-        for (Entry<String, Map<String, Boolean>> entry
-                 : roleAbilities.entrySet()) {
-            Role role = spec.getRole(entry.getKey());
-            Map<String, Boolean> required = role.getRequiredAbilities();
-            expectAbilities = entry.getValue();
-            for (String key : required.keySet()) {
-                Boolean req = required.get(key);
-                Boolean val = expectAbilities.get(key);
-                assertNotNull(role.getId() + " missing " + key, val);
-                assertEquals(role.getId() + " " + key + " value != " + req,
-                    req, val);
-                expectAbilities.remove(key);
-            }
-            assertEquals(role.getId() + " excess abilities", 0,
-                expectAbilities.size());
-        }
+        forEachMapEntry(roleAbilities, e -> {
+                Role role = spec.getRole(e.getKey());
+                Map<String, Boolean> required = role.getRequiredAbilities();
+                Map<String, Boolean> expect = e.getValue();
+                for (String key : required.keySet()) {
+                    Boolean req = required.get(key);
+                    Boolean val = expect.get(key);
+                    assertNotNull(role.getId() + " missing " + key, val);
+                    assertEquals(role.getId() + " " + key + " value != " + req,
+                                 req, val);
+                    expect.remove(key);
+                }
+                assertEquals(role.getId() + " excess abilities", 0,
+                             expect.size());
+            });
 
         Role role;
         role = spec.getRole("model.role.default");
         assertNotNull(role);
-        checkGoods(role.getId(), role.getRequiredGoods());
+        checkGoods(role.getId(), role.getRequiredGoodsList());
 
         role = spec.getRole("model.role.scout");
         assertNotNull(role);
-        checkGoods(role.getId(), role.getRequiredGoods(),
+        checkGoods(role.getId(), role.getRequiredGoodsList(),
             new AbstractGoods(horsesType, 50));
 
         role = spec.getRole("model.role.soldier");
         assertNotNull(role);
-        checkGoods(role.getId(), role.getRequiredGoods(),
+        checkGoods(role.getId(), role.getRequiredGoodsList(),
             new AbstractGoods(musketsType, 50));
 
         role = spec.getRole("model.role.dragoon");
         assertNotNull(role);
-        checkGoods(role.getId(), role.getRequiredGoods(),
+        checkGoods(role.getId(), role.getRequiredGoodsList(),
             new AbstractGoods(horsesType, 50),
             new AbstractGoods(musketsType, 50));
 
         role = spec.getRole("model.role.pioneer");
         assertNotNull(role);
-        checkGoods(role.getId(), role.getRequiredGoods(),
+        checkGoods(role.getId(), role.getRequiredGoodsList(),
             new AbstractGoods(toolsType, 20));
 
         role = spec.getRole("model.role.missionary");
         assertNotNull(role);
-        checkGoods(role.getId(), role.getRequiredGoods());
+        checkGoods(role.getId(), role.getRequiredGoodsList());
 
         role = spec.getRole("model.role.infantry");
         assertNotNull(role);
-        checkGoods(role.getId(), role.getRequiredGoods(),
+        checkGoods(role.getId(), role.getRequiredGoodsList(),
             new AbstractGoods(musketsType, 50));
 
         role = spec.getRole("model.role.cavalry");
         assertNotNull(role);
-        checkGoods(role.getId(), role.getRequiredGoods(),
+        checkGoods(role.getId(), role.getRequiredGoodsList(),
             new AbstractGoods(horsesType, 50),
             new AbstractGoods(musketsType, 50));
 
         role = spec.getRole("model.role.mountedBrave");
         assertNotNull(role);
-        checkGoods(role.getId(), role.getRequiredGoods(),
+        checkGoods(role.getId(), role.getRequiredGoodsList(),
             new AbstractGoods(horsesType, 25));
 
         role = spec.getRole("model.role.armedBrave");
         assertNotNull(role);
-        checkGoods(role.getId(), role.getRequiredGoods(),
+        checkGoods(role.getId(), role.getRequiredGoodsList(),
             new AbstractGoods(musketsType, 25));
 
         role = spec.getRole("model.role.nativeDragoon");
         assertNotNull(role);
-        checkGoods(role.getId(), role.getRequiredGoods(),
+        checkGoods(role.getId(), role.getRequiredGoodsList(),
             new AbstractGoods(horsesType, 25),
             new AbstractGoods(musketsType, 25));
     }
@@ -369,14 +372,18 @@ public final class SpecificationTest extends FreeColTestCase {
             + "</unit-types>"
             + "</freecol-specification>";
 
-        Specification spec = new Specification(new ByteArrayInputStream(specification.getBytes()));
-
+        Specification spec = null;
+        try {
+            spec = new Specification(new ByteArrayInputStream(specification.getBytes()));
+        } catch (XMLStreamException xse) {
+            fail("Spec read fail");
+        }
         assertNotNull(spec.getUnitType("model.unit.milkmaid"));
         assertNotNull(spec.getUnitType("model.unit.caravel"));
 
         // restore original values
         try {
-            spec = new Specification(new FreeColTcFile("freecol").getSpecificationInputStream());
+            spec = FreeColRules.getFreeColRulesFile("freecol").getSpecification();
         } catch (Exception e) {
             System.out.println(e);
         }
@@ -390,24 +397,18 @@ public final class SpecificationTest extends FreeColTestCase {
             + "</unit-types>"
             + "</freecol-specification>";
 
-        Specification spec = new Specification(new ByteArrayInputStream(specification.getBytes()));
-
+        Specification spec = null;
         try {
-            spec.getUnitType("model.unit.caravel");
-            fail("Caravel is defined.");
-        } catch (IllegalArgumentException e) {
+            spec = new Specification(new ByteArrayInputStream(specification.getBytes()));
+        } catch (XMLStreamException xse) {
+            fail("Spec read fail");
         }
+        assertNull("Caravel should be undefined",
+                   spec.getUnitType("model.unit.caravel"));
 
         for (UnitType unitType : spec.getUnitTypeList()) {
             assertFalse("model.unit.caravel".equals(unitType.getId()));
         }
-
-        // restore original values
-        try {
-            spec = new Specification(new FreeColTcFile("freecol").getSpecificationInputStream());
-        } catch (Exception e) {
-            System.out.println(e);
-        }
     }
 
     public void testGetDefaultUnitType() {
@@ -437,16 +438,16 @@ public final class SpecificationTest extends FreeColTestCase {
         
     public void testLoadMods() {
         try {
-            Specification specification = new Specification(new FreeColTcFile("freecol").getSpecificationInputStream());
-            int numberOfUnitTypes = specification.getUnitTypeList().size();
+            Specification spec = FreeColRules.getFreeColRulesFile("freecol")
+                .getSpecification();
+            int numberOfUnitTypes = spec.getUnitTypeList().size();
             List<FreeColModFile> mods = new ArrayList<>();
             mods.add(new FreeColModFile(new File("data/mods/example")));
-            specification.loadMods(mods);
-            UnitType milkmaid = specification.getUnitType("model.unit.milkmaid");
-            assertEquals(numberOfUnitTypes + 1, 
-                specification.getUnitTypeList().size());
-        } catch (Exception e) {
-            fail(e.getMessage());
+            spec.loadMods(mods);
+            UnitType milkmaid = spec.getUnitType("model.unit.milkmaid");
+            assertEquals(numberOfUnitTypes + 1, spec.getUnitTypeList().size());
+        } catch (IOException|XMLStreamException ex) {
+            fail(ex.toString());
         }
     }
 }
