@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2015  The FreeCol Team
+ *  Copyright (C) 2002-2022  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -19,8 +19,11 @@
 
 package net.sf.freecol.util.test;
 
-import java.lang.reflect.Field;
+import static net.sf.freecol.common.util.CollectionUtils.find;
+import static net.sf.freecol.common.util.CollectionUtils.matchKey;
+import static net.sf.freecol.common.util.CollectionUtils.none;
 
+import java.lang.reflect.Field;
 import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.List;
@@ -29,7 +32,8 @@ import java.util.Locale;
 import junit.framework.TestCase;
 import net.sf.freecol.FreeCol;
 import net.sf.freecol.common.i18n.Messages;
-import net.sf.freecol.common.io.FreeColTcFile;
+import net.sf.freecol.common.io.FreeColModFile;
+import net.sf.freecol.common.io.FreeColRules;
 import net.sf.freecol.common.model.AbstractGoods;
 import net.sf.freecol.common.model.Colony;
 import net.sf.freecol.common.model.CombatModel;
@@ -47,7 +51,9 @@ import net.sf.freecol.common.model.Specification;
 import net.sf.freecol.common.model.Tile;
 import net.sf.freecol.common.model.TileType;
 import net.sf.freecol.common.model.Unit;
+import net.sf.freecol.common.model.UnitChangeType;
 import net.sf.freecol.common.model.UnitType;
+import net.sf.freecol.common.model.UnitTypeChange;
 import net.sf.freecol.common.model.WorkLocation;
 import net.sf.freecol.server.model.ServerGame;
 import net.sf.freecol.server.model.ServerIndianSettlement;
@@ -61,6 +67,14 @@ import net.sf.freecol.server.model.ServerUnit;
  */
 public class FreeColTestCase extends TestCase {
 
+    /** We now have lots of maps, restrict testing to just the old ones. */
+    public static String[] STANDARD_MAPS = {
+        "data/maps/M_Africa_Gilolat.fsm",
+        "data/maps/S_AustraliaOceania_Gilolat.fsm",
+        "data/maps/L_America_JsTheDude.fsm",
+        "data/maps/S_Caribbean_Phil.fsm"
+    };
+        
     private static java.util.Map<String, Specification> specifications
         = new HashMap<>();
 
@@ -73,6 +87,7 @@ public class FreeColTestCase extends TestCase {
 
     @Override
     protected void setUp() throws Exception {
+        FreeColRules.loadRules();
         if (updateLocale) {
             updateLocale = false;
             Messages.loadMessageBundle(Locale.US);
@@ -125,9 +140,10 @@ public class FreeColTestCase extends TestCase {
     public static Specification getSpecification(String name) {
         Specification result = specifications.get(name);
         if (result == null) {
+            FreeColRules.loadRules();
             try {
-                FreeColTcFile tc = new FreeColTcFile(name);
-                result = FreeCol.loadSpecification(tc, null, "model.difficulty.medium");
+                FreeColModFile rules = FreeColRules.getFreeColRulesFile(name);
+                result = FreeCol.loadSpecification(rules, null, "model.difficulty.medium");
                 specifications.put(name, result);
             } catch (Exception e) {
                 e.printStackTrace();
@@ -155,7 +171,7 @@ public class FreeColTestCase extends TestCase {
      * As a side effect this call will reset the singleton game value that can
      * be accessed using getGame().
      *
-     * @param specName a <code>String</code> value
+     * @param specName a {@code String} value
      * @return A new game with with players for each nation added.
      */
     public static Game getStandardGame(String specName) {
@@ -170,7 +186,7 @@ public class FreeColTestCase extends TestCase {
         specification.applyDifficultyLevel("model.difficulty.medium");
         for (Nation n : specification.getNations()) {
             if (n.isUnknownEnemy()) continue;
-            Player p = new ServerPlayer(game, false, n, null, null);
+            Player p = new ServerPlayer(game, false, n);
             boolean ai = !n.getType().isEuropean() || n.getType().isREF();
             p.setAI(ai);
             if (ai || game.canAddNewPlayer()) game.addPlayer(p);
@@ -269,14 +285,14 @@ public class FreeColTestCase extends TestCase {
         int landWidth = (int) Math.floor(totalWidth/2);
         for (int x = 0; x < landWidth; x++) {
             for (int y = 0; y < totalHeight; y++) {
-                builder.setTile(x, y, landTileType);
+                builder.setTileType(x, y, landTileType);
             }
         }
 
         // Add high seas.
         final TileType highSeasType = spec().getTileType("model.tile.highSeas");
         for (int y = 0; y < totalHeight; y++) {
-            builder.setTile(totalWidth - 1, y, highSeasType);
+            builder.setTileType(totalWidth - 1, y, highSeasType);
         }
 
         return builder.build();
@@ -285,7 +301,7 @@ public class FreeColTestCase extends TestCase {
     /**
      * Get a standard colony at the location 5,8 with one free colonist
      *
-     * @return The <code>Colony</code> as specified.
+     * @return The {@code Colony} as specified.
      */
     public Colony getStandardColony() {
         return getStandardColony(1, 5, 8);
@@ -296,7 +312,7 @@ public class FreeColTestCase extends TestCase {
      *
      * @param numberOfSettlers The number of settlers to put into the
      *     colony.  Must be >= 1.
-     * @return The <code>Colony</code> as specified.
+     * @return The {@code Colony} as specified.
      */
     public Colony getStandardColony(int numberOfSettlers) {
         return getStandardColony(numberOfSettlers, 5, 8);
@@ -310,7 +326,7 @@ public class FreeColTestCase extends TestCase {
      *     colony.  Must be >= 1.
      * @param tileX Coordinate of tile for the colony.
      * @param tileY Coordinate of tile for the colony.
-     * @return The <code>Colony</code> as specified.
+     * @return The {@code Colony} as specified.
      */
     public Colony getStandardColony(int numberOfSettlers,
                                     int tileX, int tileY) {
@@ -331,19 +347,15 @@ public class FreeColTestCase extends TestCase {
      * Useful utility to make sure a work location is empty before doing
      * some test that implicates it.
      *
-     * @param wl The <code>WorkLocation</code> to clear.
+     * @param wl The {@code WorkLocation} to clear.
      * @return True if the work location is clear, false if there was a problem
      *     removing a unit.
      */
     public boolean clearWorkLocation(WorkLocation wl) {
         for (Unit u : wl.getUnitList()) {
-            for (WorkLocation w : wl.getColony().getCurrentWorkLocations()) {
-                if (w == wl) continue;
-                if (w.canAdd(u)) {
-                    u.setLocation(w);
-                    if (u.getLocation() == w) break;
-                }
-            }
+            WorkLocation w = find(wl.getColony().getCurrentWorkLocations(),
+                                  w2 -> w2 != wl && w2.canAdd(u));
+            if (w != null) u.setLocation(w);
         }
         return wl.isEmpty();
     }
@@ -353,7 +365,7 @@ public class FreeColTestCase extends TestCase {
         // Required parameter
         private final Game game;
 
-        private TileType[][] tiles = null;
+        private TileType[][] tileTypes = null;
         private int width;
         private int height;
         private TileType baseTile;
@@ -372,12 +384,12 @@ public class FreeColTestCase extends TestCase {
             exploredByAll = false;
             initiated = false;
             // set empty grid
-            if(tiles == null){
-                tiles = new TileType[width][height];
+            if(tileTypes == null){
+                tileTypes = new TileType[width][height];
             }
             for (int x = 0; x < width; x++) {
                 for (int y = 0; y < height; y++) {
-                    tiles[x][y] = null;
+                    tileTypes[x][y] = null;
                 }
             }
         }
@@ -392,17 +404,17 @@ public class FreeColTestCase extends TestCase {
 
         public MapBuilder setDimensions(int width, int heigth){
             if(width <= 0){
-                throw new IllegalArgumentException("Width must be positive");
+                throw new RuntimeException("Width must be positive");
             }
             if(heigth <= 0){
-                throw new IllegalArgumentException("Heigth must be positive");
+                throw new RuntimeException("Heigth must be positive");
             }
             if(initiated){
                 throw new IllegalStateException("Cannot resize map after setting a tile");
             }
             this.width = width;
             this.height = heigth;
-            this.tiles = new TileType[width][height];
+            this.tileTypes = new TileType[width][height];
             return this;
         }
 
@@ -411,18 +423,18 @@ public class FreeColTestCase extends TestCase {
             return this;
         }
 
-        public MapBuilder setTile(int x, int y, TileType tileType){
+        public MapBuilder setTileType(int x, int y, TileType tileType){
             if(x < 0 || y < 0){
-                throw new IllegalArgumentException("Coordenates cannot be negative");
+                throw new RuntimeException("Coordenates cannot be negative");
             }
             if(x >= width || y >= height ){
-                throw new IllegalArgumentException("Coordenate out of bounds");
+                throw new RuntimeException("Coordenate out of bounds");
             }
             if(tileType == null){
                 throw new NullPointerException("Tile type cannot be null");
             }
 
-            tiles[x][y]= tileType;
+            tileTypes[x][y]= tileType;
             initiated = true;
 
             return this;
@@ -434,10 +446,10 @@ public class FreeColTestCase extends TestCase {
             for (int x = 0; x < width; x++) {
                 for (int y = 0; y < height; y++) {
                     // Already manually set by the tester
-                    if(tiles[x][y] != null){
+                    if(tileTypes[x][y] != null){
                         continue;
                     }
-                    tiles[x][y] = baseTile;
+                    tileTypes[x][y] = baseTile;
                 }
             }
             initiated=true;
@@ -448,23 +460,21 @@ public class FreeColTestCase extends TestCase {
             Map map = new Map(game, width, height);
             Region region = new Region(game);
 
-            for (int x = 0; x < width; x++) {
-                for (int y = 0; y < height; y++) {
-                    TileType tileType = tiles[x][y];
+            map.populateTiles((x, y) -> {
+                    TileType tileType = tileTypes[x][y];
                     Tile t = new Tile(game, tileType, x, y);
                     t.setRegion(region);
-                    map.setTile(t, x, y);
-                }
-            }
+                    return t;
+                });
 
             map.resetContiguity();
             map.resetHighSeasCount();
             if (exploredByAll) {
-                for (Tile tile : map.getAllTiles()) {
-                    for (Player player : game.getLiveEuropeanPlayers(null)) {
-                        tile.setExplored(player, true);
-                    }
-                }
+                map.forEachTile(t -> {
+                        for (Player p : game.getLiveEuropeanPlayerList()) {
+                            t.setExplored(p, true);
+                        }
+                    });
             }
             return map;
         }
@@ -478,11 +488,12 @@ public class FreeColTestCase extends TestCase {
 
     public static class IndianSettlementBuilder{
 
+        private static final String defaultIndianPlayer = "model.nation.tupi";
+
         // Required parameter
         private final Game game;
 
         private Player indianPlayer;
-        private final String defaultIndianPlayer = "model.nation.tupi";
         private String skillTaught;
         private int initialBravesInCamp;
         private Tile settlementTile;
@@ -491,16 +502,17 @@ public class FreeColTestCase extends TestCase {
         private boolean isCapital;
         private Unit residentMissionary;
 
-        public IndianSettlementBuilder(Game game){
+
+        public IndianSettlementBuilder(Game game) {
             this.game = game;
             setStartingParams();
         }
 
-        private void setStartingParams(){
-            // Some params can only be set in build(), because the default values
-            //may not be valid for the game set
-            // However, the tester himself may set them to valid values later,
-            //so they are set to null for now
+        private void setStartingParams() {
+            // Some params can only be set in build(), because the
+            // default values may not be valid for the game set
+            // However, the tester himself may set them to valid values
+            // later, so they are set to null for now
             indianPlayer = null;
             initialBravesInCamp = 1;
             settlementTile = null;
@@ -509,46 +521,46 @@ public class FreeColTestCase extends TestCase {
             residentMissionary = null;
         }
 
-        public IndianSettlementBuilder player(Player player){
-            this.indianPlayer = player;
-
-            if(indianPlayer == null || !game.getPlayers().contains(player)){
-                throw new IllegalArgumentException("Indian player not in game");
+        public IndianSettlementBuilder player(Player player) {
+            if (player == null
+                || none(game.getPlayers(matchKey(player)))) {
+                throw new RuntimeException("Indian player not in game");
             }
-
+            this.indianPlayer = player;
             return this;
         }
 
-        public IndianSettlementBuilder initialBravesInCamp(int nBraves){
-            if(nBraves <= 0){
-                throw new IllegalArgumentException("Number of braves must be positive");
+        public IndianSettlementBuilder initialBravesInCamp(int nBraves) {
+            if (nBraves <= 0) {
+                throw new RuntimeException("Number of braves must be positive");
             }
             this.initialBravesInCamp = nBraves;
             return this;
         }
 
-        public IndianSettlementBuilder settlementTile(Tile tile){
-            Tile tileOnMap = this.game.getMap().getTile(tile.getX(), tile.getY());
-            if(tile != tileOnMap){
-                throw new IllegalArgumentException("Given tile not on map");
+        public IndianSettlementBuilder settlementTile(Tile tile) {
+            Tile tileOnMap = this.game.getMap()
+                .getTile(tile.getX(), tile.getY());
+            if (tile != tileOnMap) {
+                throw new RuntimeException("Given tile not on map");
             }
             this.settlementTile = tile;
             return this;
         }
 
-        public IndianSettlementBuilder capital(boolean isCapital){
+        public IndianSettlementBuilder capital(boolean isCapital) {
             this.isCapital = isCapital;
 
             return this;
         }
 
-        public IndianSettlementBuilder missionary(Unit missionary){
+        public IndianSettlementBuilder missionary(Unit missionary) {
             this.residentMissionary = missionary;
 
             return this;
         }
 
-        public IndianSettlementBuilder skillToTeach(String skill){
+        public IndianSettlementBuilder skillToTeach(String skill) {
             this.skillTaught = skill;
 
             return this;
@@ -559,41 +571,43 @@ public class FreeColTestCase extends TestCase {
                 : "Settlement-" + settlementNumber++;
         }
 
-        public IndianSettlement build(){
+        public IndianSettlement build() {
             UnitType skillToTeach = null;
 
-            if(skillTaught != null){
+            if (skillTaught != null) {
                 skillToTeach = spec().getUnitType(skillTaught);
             }
 
             // indianPlayer not set, get default
             if (indianPlayer == null) {
                 indianPlayer = game.getPlayerByNationId(defaultIndianPlayer);
-                if(indianPlayer == null){
-                    throw new IllegalArgumentException("Default Indian player " + defaultIndianPlayer + " not in game");
+                if (indianPlayer == null) {
+                    throw new RuntimeException("Default Indian player "
+                        + defaultIndianPlayer + " not in game");
                 }
             }
             UnitType indianBraveType = spec().getDefaultUnitType(indianPlayer);
 
             // settlement tile no set, get default
-            if(settlementTile == null){
+            if (settlementTile == null) {
                 settlementTile = game.getMap().getTile(5, 8);
-                if(settlementTile == null){
-                    throw new IllegalArgumentException("Default tile not in game");
+                if (settlementTile == null) {
+                    throw new RuntimeException("Default tile not in game");
                 }
             }
 
-            IndianSettlement camp
-                = new ServerIndianSettlement(game, indianPlayer,
-                                             getSimpleName(indianPlayer, isCapital),
-                                             settlementTile, isCapital,
-                                             skillToTeach, residentMissionary);
+            IndianSettlement camp = new ServerIndianSettlement(game,
+                indianPlayer, getSimpleName(indianPlayer, isCapital),
+                settlementTile, isCapital, skillToTeach, residentMissionary);
             indianPlayer.addSettlement(camp);
 
             // Add braves
-            for(int i=0; i < initialBravesInCamp; i++){
+            for (int i = 0; i < initialBravesInCamp; i++) {
                 Unit brave = new ServerUnit(game, camp, indianPlayer,
                                             indianBraveType);
+                if (brave == null) {
+                    throw new RuntimeException("Null brave");
+                }
                 camp.addOwnedUnit(brave);
             }
             camp.placeSettlement(true);
@@ -612,17 +626,18 @@ public class FreeColTestCase extends TestCase {
      * Set the production bonus of the given colony to the given
      * value.
      *
-     * @param colony a <code>Colony</code> value
-     * @param value an <code>int</code> value
+     * @param colony a {@code Colony} value
+     * @param value an {@code int} value
      */
     public void setProductionBonus(Colony colony, int value) {
+        Field productionBonus;
         try {
-            Field productionBonus = Colony.class.getDeclaredField("productionBonus");
+            productionBonus = Colony.class.getDeclaredField("productionBonus");
             productionBonus.setAccessible(true);
             productionBonus.setInt(colony, value);
             colony.invalidateCache();
-        } catch (Exception e) {
-            // do nothing
+        } catch (NoSuchFieldException|IllegalAccessException ex) {
+            fail(ex.toString());
         }
     }
 
@@ -676,8 +691,8 @@ public class FreeColTestCase extends TestCase {
      * Check a combat result list.
      *
      * @param name A base string for the error message.
-     * @param crs The list of <code>CombatResult</code> to check.
-     * @param results The expected <code>CombatResult</code>s.
+     * @param crs The list of {@code CombatResult} to check.
+     * @param results The expected {@code CombatResult}s.
      */
     public void checkCombat(String name, List<CombatResult> crs,
                             CombatResult... results) {
@@ -706,8 +721,8 @@ public class FreeColTestCase extends TestCase {
      * Check a list of goods.
      *
      * @param err A base string for the error message.
-     * @param goods A list of expected <code>AbstractGoods</code> to check.
-     * @param results The expected <code>AbstractGoods</code>.
+     * @param goods A list of expected {@code AbstractGoods} to check.
+     * @param results The expected {@code AbstractGoods}.
      */
     public void checkGoods(String err, List<AbstractGoods> goods,
                            AbstractGoods... results) {
@@ -718,4 +733,31 @@ public class FreeColTestCase extends TestCase {
         }
         assertTrue(err + " requires more goods", check.isEmpty());
     }
+
+    public void addUnitTypeChange(String type, UnitType from, UnitType to,
+                                  int probability, int turns) {
+        UnitTypeChange utc = new UnitTypeChange(type + "."
+            + from.getSuffix() + "-" + to.getSuffix(), spec());
+        utc.from = from;
+        utc.to = to;
+        utc.probability = probability;
+        utc.turns = turns;
+        UnitChangeType uct = spec().getUnitChangeType(type);
+        if (uct == null) {
+            uct = new UnitChangeType(type, spec());
+            spec().getUnitChangeTypeList().add(uct);
+        }
+        uct.addUnitTypeChange(utc);
+    }
+
+    /**
+     * Helper function to get a server player by id.
+     *
+     * @param game The {@code Game} to look in.
+     * @param id The player identifier.
+     * @return The {@code ServerPlayer} found.
+     */
+    public ServerPlayer getServerPlayer(Game game, String id) {
+        return (ServerPlayer)game.getPlayerByNationId(id);
+    }
 }
